/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.bpel.deploy;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Collection;
import java.util.Collections;
import java.util.Map;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.hibernate.SessionFactory;
import org.hibernate.cache.Cache;
import org.hibernate.cfg.Configuration;
import org.hibernate.impl.SessionFactoryImpl;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;

/**
 * The activator class controls the plug-in life cycle
 */
public class BPELDeployActivator extends AbstractUIPlugin implements IPropertyChangeListener {

	// The plug-in ID
	public static final String PLUGIN_ID = "org.spagic3.bpel.deploy.metadb";

	// The shared instance
	private static BPELDeployActivator plugin;
	
	private SessionFactory sf = null;
	/**
	 * The constructor
	 */
	public BPELDeployActivator() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		
		
		
		getDefault().getPluginPreferences().addPropertyChangeListener(this);

	}
	
	public void propertyChange(org.eclipse.core.runtime.Preferences.PropertyChangeEvent event) {	
		this.sf = buildSessionFactory();
	
		
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static BPELDeployActivator getDefault() {
		return plugin;
	}

	public static File getFileFromPlugin(String filePath){
		Bundle bundle = getDefault().getBundle();
		Path path = new Path(filePath); 
		URL url = FileLocator.find(bundle, path, Collections.EMPTY_MAP);
		URL fileUrl = null;
		try {
			fileUrl = FileLocator.toFileURL(url);
		}catch (IOException e) {
			// Will happen if the file cannot be read for some reason
			e.printStackTrace();
		}
		return new File(fileUrl.getPath());
	}
	
	public SessionFactory buildSessionFactory(){
		Configuration cfg = new Configuration().configure();
		org.eclipse.core.runtime.Preferences pref = SpagicUtilsActivator.getDefault().getPluginPreferences();
		
		String dbUrl = pref.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_URL);
		String dbDriver = pref.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_DRIVER);
		String dbUser = pref.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_USER);
		String dbPassword = pref.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_PASSWORD);
		String dbDialect = pref.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_HIB_DIALECT);
		String dbShowSql = pref.getString(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_SHOW_SQL);
		
		cfg.setProperty("hibernate.connection.driver_class", dbDriver);
        cfg.setProperty("hibernate.connection.url",dbUrl);
        cfg.setProperty("hibernate.connection.username",dbUser);
        cfg.setProperty("hibernate.connection.password",dbPassword);
        
        cfg.setProperty("hibernate.dialect",dbDialect);
        cfg.setProperty("hibernate.show_sql",dbShowSql);
        
		return cfg.buildSessionFactory();
	}
	
	
	

		
		
		
		
		public  SessionFactory getSessionFactory(){
			if (sf == null){
//				System.out.println(" Building Session Factory");
				this.sf = buildSessionFactory();
				
			}
			debugSFAndClearCache(sf);
			return sf;
		}
		
		protected static void debugSFAndClearCache(SessionFactory sessionFactory) {
			SessionFactoryImpl sessionFactoryImpl = (SessionFactoryImpl)sessionFactory;	
			String[] persistentClasses = sessionFactoryImpl.getImplementors("java.lang.Object");
			for (String persistentClass : persistentClasses) {
				sessionFactoryImpl.evictEntity(persistentClass);
			}
			if (sessionFactoryImpl.getQueryCache() != null){
				sessionFactoryImpl.getQueryCache().clear();
			}
			sessionFactory.evictQueries();
			Map cacheRegionsMap = sessionFactoryImpl.getAllSecondLevelCacheRegions() ;
			Collection<Cache> cacheRegions = cacheRegionsMap.values();
			for (Cache cache : cacheRegions) {
//				System.out.println(" Found Cache Region");
				cache.clear();
			}
			
			
		}
}
