/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.bpel.deploy.ui.actions;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.activation.MimetypesFileTypeMap;
import javax.xml.namespace.QName;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ExtractDataRule;
import org.eclipse.stp.im.ImPackage;
import org.eclipse.stp.im.IterationControl;
import org.eclipse.stp.im.JoinControl;
import org.eclipse.stp.im.MapProperty;
import org.eclipse.stp.im.ObservableAttribute;
import org.eclipse.stp.im.ProcessCollection;
import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.RouterControl;
import org.eclipse.stp.im.Service;
import org.eclipse.stp.im.ServiceBinding;
import org.eclipse.stp.im.SplitControl;
import org.eclipse.stp.im.Step;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.Transition;
import org.eclipse.stp.im.Variable;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.spagic.metadb.base.MonitoringTypeConstants;
import org.spagic.metadb.base.ProcessStateConstants;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.Attribute;
import org.spagic.metadb.model.MonitoringType;
import org.spagic.metadb.model.Process;
import org.spagic.metadb.model.ProcessDiagram;
import org.spagic.metadb.model.PropertyProcess;
import org.spagic.metadb.model.PropertyProcessId;
import org.spagic.metadb.model.PropertyStep;
import org.spagic.metadb.model.PropertyStepId;
import org.spagic.metadb.model.Rule;
import org.spagic.metadb.model.SubProcess;
import org.spagic.metadb.model.SubProcessId;
import org.spagic.metadb.model.TransitionId;
import org.spagic.metadb.model.TypeService;
import org.spagic3.bpel.deploy.BPELDeployActivator;
import org.spagic3.bpel.deploy.metadb.processor.IProcessor;
import org.spagic3.bpel.deploy.metadb.processor.RuntimeProcessor;
import org.spagic3.utils.SpagicLogger;
import org.spagic3.utils.SpagicUtils;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.graph.GraphGenerator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;

public class Im2MetaDBWorkspaceOperation extends WorkspaceModifyOperation {

	public static final Integer USAGE_EXTRACT_DATA = 0;
	public static final Integer USAGE_ITER_TYPE_DETERMINATION = 1;

	public static final String RULE_DESCRIPTION = "RuleForIterTypeDetermination:";
	public static final String ITER_DESCRIPTION = "No description available";
	public static final SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
	private IFile imFile = null;


	// Saint - Thu Dec 24 15:05:50 2009
	// 
	// Patterns to scan Orbeon form definition chasing relevant data
//	private final static Pattern relevantValueHunterPattern =
//		Pattern.compile("<xforms:input.*class=\"[^\"]*fr-search[^\"]*?\"[^>]*>");
//
//	private final static Pattern relevantValueNameExtractorPattern = 
//		Pattern.compile("id=\"(.+)-control\"");
	//  </Saint>

	public IFile getImFile() {
		return imFile;
	}

	public void setImFile(IFile imFile) {
		this.imFile = imFile;
	}

	public Im2MetaDBWorkspaceOperation(IFile imFile) {
		this.imFile = imFile;
	}

	/**
	 * Open the Session Factory retrieving the parameters from Spagic Preference
	 * page
	 * 
	 * @return A new Session Factory
	 */
	public SessionFactory getSessionFactory() {
		return BPELDeployActivator.getDefault().getSessionFactory();
	}

	protected void execute(IProgressMonitor monitor) {

		Session aSession = null;
		Transaction tx = null;

		try {
			// Retrieve the Intermediate Model that the user selected
			StpIntermediateModel imModel = retrieveIMModel();

			// Begin Hibernate transaction
			aSession = getSessionFactory().openSession();
			tx = aSession.beginTransaction();

			// Save all processes in process collection
			ProcessCollection processCollection = imModel.getProcessCollection();
			List<org.eclipse.stp.im.Process> processList = processCollection.getProcesses();
			List<org.spagic.metadb.model.Process> metaDBProcessList = new ArrayList<org.spagic.metadb.model.Process>(
					processList.size());
			for (org.eclipse.stp.im.Process process : processList) {

				// Save the process and all sub objects
				org.spagic.metadb.model.Process metaDBProcess = saveEntireProcess(aSession, process, imModel);
				metaDBProcessList.add(metaDBProcess);

			}

			// Save iter
			saveIter(aSession, imModel, metaDBProcessList);

			// Force writing data on database
			aSession.flush();

			// Produce all graphs
			org.spagic.metadb.model.Process metaDBProcess = null;
			Iterator<org.spagic.metadb.model.Process> metaDBProcessIt = metaDBProcessList.iterator();
			for (org.eclipse.stp.im.Process process : processList) {
				metaDBProcess = metaDBProcessIt.next();

				// Create the graph and save it on the metadb
				saveProcessGraph(aSession, process, metaDBProcess);
			}

			// Force writing data on database
			aSession.flush();
			tx.commit();

		} catch (IllegalArgumentException iaex) {
			if (tx != null) {
				tx.rollback();
			}
			SpagicLogger.error(iaex.getMessage(), iaex);
			throw new RuntimeException("Error Publishing Intermediate Model In Metadatabase", iaex);

		} catch (Exception e) {
			if (tx != null) {
				tx.rollback();
			}
			SpagicLogger.error(e.getMessage(), e);

			throw new RuntimeException("Error Publishing Intermediate Model In Metadatabase", e);
		} finally {
			if (aSession != null) {
				aSession.close();
			}
		}
	}

	/**
	 * @return The location of the Graphviz executable.
	 */
	public String getGraphVizExeLocation() {
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		return prefs.getString(SpagicPreferenceConstants.P_GRAPH_VIZ_DOT_EXE);
	}

	/**
	 * @return The location of the temporary folder used for generating graphs.
	 */
	public String getGraphVizTempFolder() {
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		return prefs.getString(SpagicPreferenceConstants.P_GRAPH_VIZ_TEMP_FOLDER);
	}

	/**
	 * Save a process on meta-db and all related sub-objects (steps, transitions,
	 * etc).
	 * 
	 * @param aSession
	 *          Hibernate session
	 * @param process
	 *          Intermediate Model process to save
	 * @return The saved process
	 */
	private org.spagic.metadb.model.Process saveEntireProcess(Session aSession, org.eclipse.stp.im.Process process,
			StpIntermediateModel stpIntermediateModel) {
		// Retrieve the processor to apply to this process before saving on the
		// database
		// TODO Dialog richiesta scelta binding
		String runtimeBinding = retrieveBindingForProcess(process);
		IProcessor processor = RuntimeProcessor.getInstance().getProcessor(runtimeBinding);

		// Apply the processor to the process: this can change the process, but if
		// we dont save the process, the changes are not persisted on the IM (that
		// is
		// the desired behaviour)
		process = processor.preProcess(imFile, process, stpIntermediateModel);

		// Save the process
		org.spagic.metadb.model.Process metaDBProcess = saveProcess(aSession, process, processor);

		// Retrieve the transitions
		List<Transition> transitionList = process.getTransitions();

		// Save the steps
		List<Step> stepList = process.getSteps();
		Map<String, org.spagic.metadb.model.Step> stepMap = new HashMap<String, org.spagic.metadb.model.Step>();
		for (Step step : stepList) {
			// Save the step
			/*
			 * boolean isStartStep = isStartStep(transitionList, step); boolean
			 * isEndStep = isEndStep(transitionList, step);
			 */
			org.spagic.metadb.model.Step metaDBStep = saveStep(aSession, step, SpagicUtils.isStartStep(step), SpagicUtils
					.isEndStep(step), process, metaDBProcess, runtimeBinding);
			stepMap.put(new QName(metaDBStep.getNamespace(), metaDBStep.getName()).toString(), metaDBStep);
		}

		aSession.flush();

		// Saves the transitions
		for (Transition transition : transitionList) {
			saveTransition(aSession, transition, process, metaDBProcess, stepMap);
		}

		return metaDBProcess;
	}

	/**
	 * Retrieve the Intermediate Model that the user selected.
	 * 
	 * @return The Intermediate Model that the user selected.
	 */
	private StpIntermediateModel retrieveIMModel() {
		// Read the bpmn emf model
		ResourceSet resourceSetIM = new ResourceSetImpl();
		URI uri = URI.createPlatformResourceURI(imFile.getFullPath().toString(), true);

		// Force Package Registration
		@SuppressWarnings("unused")
		ImPackage imPackage = ImPackage.eINSTANCE;

		// load on demand = true
		Resource resourceIM = resourceSetIM.getResource(uri, true);

		// Get The intermediate model
		StpIntermediateModel imModel = (StpIntermediateModel) resourceIM.getContents().get(0);
		return imModel;
	}

	private String retrieveBindingForProcess(org.eclipse.stp.im.Process process) {

		Map<String, String> usedRuntimes = SpagicUtils.getRuntimeUsedByProcess(process);
		String choosedRuntimeId = null;

		if (usedRuntimes.values().size() == 1) {

			choosedRuntimeId = usedRuntimes.keySet().iterator().next();
			// MessageDialog.openInformation(new Shell(), "Spagic",
			// "Process ["+process.getName()+"] Only ["+usedRuntimes.values().iterator().next()+"] Runtime Found");

		} else {

			throw new RuntimeException(" No Runtime Found For Process ");
		}

		return choosedRuntimeId;

	}

	/**
	 * Retrieve the binding used by the process
	 * 
	 * @param process
	 *          Process to store on database
	 * @return The binding used in the process
	 * 
	 *         private String retrieveBindingForProcess(org.eclipse.stp.im.Process
	 *         process) {
	 * 
	 *         Map<String, String> usedRuntimes =
	 *         SpagicUtils.getRuntimeUsedByProcess(process); String
	 *         choosedRuntimeId = null;
	 * 
	 *         if (usedRuntimes.values().size() == 1){
	 * 
	 *         choosedRuntimeId = usedRuntimes.keySet().iterator().next();
	 *         //MessageDialog.openInformation(new Shell(), "Spagic",
	 *         "Process ["+process
	 *         .getName()+"] Only ["+usedRuntimes.values().iterator
	 *         ().next()+"] Runtime Found");
	 * 
	 *         }else{
	 * 
	 *         ChooseRuntimeDialog aChooseRuntimeDialog = new
	 *         ChooseRuntimeDialog(new Shell(),usedRuntimes);
	 *         aChooseRuntimeDialog.open(); choosedRuntimeId =
	 *         aChooseRuntimeDialog.getChoosedRuntimeId(); }
	 * 
	 *         return choosedRuntimeId;
	 * 
	 *         }
	 */

	/**
	 * Determine if a step is a starting step.
	 * 
	 * @param transitionList
	 *          List of all process transitions
	 * @param currentStep
	 *          Step to examine
	 * @return True if the step is a starting step in the process
	 */
	/*
	 * private boolean isStartStep(List<Transition> transitionList, Step
	 * currentStep) { for (Transition transition : transitionList) { Step
	 * targetStep = transition.getTarget(); if (targetStep.equals(currentStep)) {
	 * return false; } } return true; }
	 */

	/**
	 * Determine if a step is an ending step.
	 * 
	 * @param transitionList
	 *          List of all process transitions
	 * @param currentStep
	 *          Step to examine
	 * @return True if the step is an ending step in the process
	 */
	/*
	 * private boolean isEndStep(List<Transition> transitionList, Step
	 * currentStep) { for (Transition transition : transitionList) { Step
	 * sourceStep = transition.getSource(); if (sourceStep.equals(currentStep)) {
	 * return false; } } return true; }
	 */

	/**
	 * Saves on the meta-db without sub-objects.
	 * 
	 * @param aSession
	 *          Hibernate session
	 * @param process
	 *          Process to save
	 * @return The saved process
	 */
	private org.spagic.metadb.model.Process saveProcess(Session aSession, org.eclipse.stp.im.Process process,
			IProcessor processor) {
		String processName = process.getName();
		String processNamespace = null;
		String processVersion = null;

		// Retrieve process name and version to check that the same process doesn't
		// exist already in the database
		org.spagic.metadb.model.Process existingProcess = null;
		{
			Property propertyVersion = process.getProperties().get(ImConstants.PROCESS_VERSION);
			processVersion = (String) propertyVersion.getValue();

			if (process.getProperties().get(ImConstants.PROPERTY_NAMESPACE) != null)
				processNamespace = (String)process.getProperties().get(ImConstants.PROPERTY_NAMESPACE).getValue();
			else
				processNamespace = SpagicUtils.getNameSpaceForProcess(process);

			existingProcess = AuditDBUtils.getProcessByQNameAndVersion(aSession, new QName(processNamespace, processName),
					processVersion);
			if (existingProcess != null) {
				if (!processor.handleAutomaticVersioning()) {
					throw new IllegalArgumentException("Process identified by namespace " + processNamespace + ", name "
							+ processName + ", and version" + processVersion + " already exists in database");
				}
			}
		}

		// Retrieve the monitoring type for this process
		Property propertyMonitoringType = process.getProperties().get(ImConstants.MONITORING_TYPE);
		String monitoringType = (propertyMonitoringType != null) ? (String) propertyMonitoringType.getValue()
				: ImConstants.MONITORING_TYPE_ASYNCHRONOUS;

		// Create a new process
		org.spagic.metadb.model.Process metaDBProcess = new org.spagic.metadb.model.Process();

		metaDBProcess.setName(processName);
		metaDBProcess.setProcessVersion(processVersion);
		if ((existingProcess != null) && processor.handleAutomaticVersioning()) {
			// 
			// If for the given technology we can automatically make a version
			//
			metaDBProcess.setProcessVersion(sdf.format(new Date()));
		}

		metaDBProcess.setNamespace(processNamespace);
		metaDBProcess.setDescription(process.getDescription());
		TypeService ts = AuditDBUtils.getTypeServiceWithValue(aSession, processor.getTechnology() + "-PROCESS");
		metaDBProcess.setTypeService(ts);
		// TODO
		// metaDBProcess.setUddiName(uddiName);
		// metaDBProcess.setUddiOrg(uddiOrg);
		metaDBProcess.setStartdate(new Date());
		metaDBProcess.setEnddate(null);
		metaDBProcess.setState(ProcessStateConstants.PROCESS_STARTED);
		metaDBProcess.setMonitorEnabled(true);
		metaDBProcess.setMonitoringType(new MonitoringType(
				monitoringType.equals(ImConstants.MONITORING_TYPE_ASYNCHRONOUS) ? MonitoringTypeConstants.ASYNCHRONOUS
						: MonitoringTypeConstants.SYNCHRONOUS));
		aSession.save(metaDBProcess);

		// Store all other properties
		for (Map.Entry<String, Property> entry : process.getProperties()) {
			Property property = entry.getValue();
			if (!(entry.getKey().startsWith("ProcessParameters["))) {
				if (property.getValue() instanceof String) {

					org.spagic.metadb.model.Property metaDBProperty = new org.spagic.metadb.model.Property();

					metaDBProperty.setCode(property.getKey());
					metaDBProperty.setValue((String) property.getValue());
					aSession.save(metaDBProperty);

					// Associate Property and Process
					PropertyProcessId propertyProcessId = new PropertyProcessId(metaDBProperty.getIdProperty(), metaDBProcess
							.getIdProcess());
					PropertyProcess metaDBPropertyProcess = new PropertyProcess(propertyProcessId, metaDBProcess, metaDBProperty);
					aSession.save(metaDBPropertyProcess);
				}
			}
		}

		// -------------------------------------
		// Store ProcessParameters Attributes
		// --------------------------------------

		List<String> justInsertedProperties = new ArrayList<String>();
		for (Map.Entry<String, Property> entry : process.getProperties()) {
			String entryKey = entry.getKey();
//			Property property = entry.getValue();

			if (entryKey.startsWith("ProcessParameters[")) {
				String idxMapPropertyString = entryKey.substring(entryKey.indexOf("[") + 1, entryKey.indexOf("]"));

				String code = (String) process.getProperties().get("ProcessParameters[" + idxMapPropertyString + "].Name")
				.getValue();
				String value = (String) process.getProperties().get("ProcessParameters[" + idxMapPropertyString + "].Value")
				.getValue();

				if (!(justInsertedProperties.contains(code))) {
					org.spagic.metadb.model.Property metaDBProperty = new org.spagic.metadb.model.Property();

					metaDBProperty.setCode(code);
					metaDBProperty.setValue(value);
					aSession.save(metaDBProperty);

					// Associate Property and Process
					PropertyProcessId propertyProcessId = new PropertyProcessId(metaDBProperty.getIdProperty(), metaDBProcess
							.getIdProcess());
					PropertyProcess metaDBPropertyProcess = new PropertyProcess(propertyProcessId, metaDBProcess, metaDBProperty);
					aSession.save(metaDBPropertyProcess);
					justInsertedProperties.add(code);
				}

			}
		}

		// Store Process Variables
		for (Variable aVariable : process.getProcessVariables()) {

			org.spagic.metadb.model.Property metaDBProperty = new org.spagic.metadb.model.Property();

			// TODO
			// Verify if it can be useful decouple the STP IM property names and the
			// MetaDB property names

			String varName = aVariable.getName();
			String varValue = "java.lang.String()";
			if (varName.indexOf("->") > 0) {
				// The variable has a predefined value
				String[] splitted = varName.split("->");
				varName = splitted[0];
				varValue = splitted[1];

			}

			metaDBProperty.setCode("Var[" + varName + "]");
			metaDBProperty.setValue(varValue);
			aSession.save(metaDBProperty);

			// Associate Property and Process
			PropertyProcessId propertyProcessId = new PropertyProcessId(metaDBProperty.getIdProperty(), metaDBProcess.getIdProcess());
			PropertyProcess metaDBPropertyProcess = new PropertyProcess(propertyProcessId, metaDBProcess, metaDBProperty);
			aSession.save(metaDBPropertyProcess);

			//Create Variable Definition
			org.spagic.metadb.model.VariableDefinition definition = new org.spagic.metadb.model.VariableDefinition(metaDBProcess, aVariable.getName());
			BasicProperty varSchemaBasicProperty = (BasicProperty)process.getProperties().get("Schema[" + varName + "]");
			if ( varSchemaBasicProperty != null)
				definition.setVarSchema(varSchemaBasicProperty.getValue());
			aSession.save(definition);
		}

		return metaDBProcess;
	}

	private void createMetaDBPropertyForStep(Session aSession, org.spagic.metadb.model.Step aMetaDbStep,
			String propertyKey, String propertyValue) {
		org.spagic.metadb.model.Property metaDBProperty = null;

		metaDBProperty = new org.spagic.metadb.model.Property();

		// TODO
		// Verify if it can be useful decouple the STP IM property names and the
		// MetaDB property names
		metaDBProperty.setCode(propertyKey);
		metaDBProperty.setValue(propertyValue);
		aSession.save(metaDBProperty);

		// Associate Property and Step
		PropertyStepId propertyStepId = new PropertyStepId(aMetaDbStep.getIdStep(), metaDBProperty.getIdProperty());
		PropertyStep metaDBPropertyStep = new PropertyStep(propertyStepId, aMetaDbStep, metaDBProperty);
		aSession.save(metaDBPropertyStep);
	}

	/**
	 * Saves a single process step
	 * 
	 * @param aSession
	 *          Hibernate session
	 * @param step
	 *          Step to save
	 * @param isStartStep
	 *          True if the step is a starting step
	 * @param isEndStep
	 *          True if the step is an ending step
	 * @param process
	 *          The step owner process
	 * @param metaDBProcess
	 *          The already saved owner process
	 * @return The savedn step
	 */
	private org.spagic.metadb.model.Step saveStep(Session aSession, org.eclipse.stp.im.Step step, boolean isStartStep,
			boolean isEndStep, org.eclipse.stp.im.Process process, org.spagic.metadb.model.Process metaDBProcess,
			String runtimeBinding) {

		org.spagic.metadb.model.Step metaDBStep = new org.spagic.metadb.model.Step();

		// Compute step QName
		QName stepQName = SpagicUtils.getQNameForStep(process, step);

		metaDBStep.setDescription(step.getDescription());
		metaDBStep.setName(stepQName.getLocalPart());
		metaDBStep.setNamespace(stepQName.getNamespaceURI());
		metaDBStep.setProcess(metaDBProcess);
		metaDBStep.setProcessVersion(metaDBProcess.getProcessVersion());
		metaDBStep.setIsStart(isStartStep);
		metaDBStep.setIsEnd(isEndStep);

		BasicProperty hasConditionalExitProperty = (BasicProperty) step.getProperties().get(
				ImConstants.HAS_STEP_CONDITIONAL_EXIT_FLAG);
		boolean hasConditionalExitFlag = (hasConditionalExitProperty != null ? Boolean.valueOf(hasConditionalExitProperty
				.getValue()) : false);

		metaDBStep.setHasConditionalExit(hasConditionalExitFlag);
		aSession.save(metaDBStep);

		boolean isSubprocess = false;
		String subProcessName = null;

		BasicProperty isSubProcessProperty = (BasicProperty) step.getProperties().get(ImConstants.IS_STEP_SUBPROCESS);
		isSubprocess = (isSubProcessProperty != null ? Boolean.valueOf(isSubProcessProperty.getValue()) : false);
		if (isSubprocess) {
			BasicProperty subProcessProperty = (BasicProperty) step.getProperties().get(ImConstants.SUBPROCESS_PROPERTY);
			if (subProcessProperty != null) {
				subProcessName = subProcessProperty.getValue();
			} else {
				throw new RuntimeException(" The Step [" + stepQName.getLocalPart()
						+ "] is marked as Subprocess but does not have the Subprocess Property");
			}
		}

		if (isSubprocess) {
			SubProcessId subProcessId = new SubProcessId();
			String currenteSubProcessVersion = AuditDBUtils.getCurrentProcessVersion(subProcessName, aSession);
			if (currenteSubProcessVersion == null) {
				throw new RuntimeException(" Not found a SubProcess Called [" + subProcessName + "]");
			}
			Process metaDBSubProcess = AuditDBUtils.getProcessByNameAndVersion(aSession, subProcessName,
					currenteSubProcessVersion);
			if (metaDBSubProcess == null) {
				throw new RuntimeException(" Not found a SubProcess Called [" + subProcessName + "]");
			}
			subProcessId.setIdProcess(metaDBProcess.getIdProcess());
			subProcessId.setIdStep(metaDBStep.getIdStep());
			subProcessId.setIdSubprocess(metaDBSubProcess.getIdProcess());
			org.spagic.metadb.model.SubProcess subProcess = new SubProcess();
			subProcess.setId(subProcessId);
			subProcess.setProcessByIdProcess(metaDBProcess);
			subProcess.setProcessByIdProcess(metaDBSubProcess);
			aSession.save(subProcess);
			aSession.flush();
		}

		for (Map.Entry<String, Property> entry : step.getProperties()) {
			Property property = entry.getValue();
			if (property.getValue() instanceof String) {
				String propertyValue = (String) property.getValue();
				if (propertyValue != null && propertyValue.trim().length() > 0)
					createMetaDBPropertyForStep(aSession, metaDBStep, property.getKey(), propertyValue);

				// modified to store documentation!

				if (property.getKey().equals("documentationRelativePath")) {
					saveStepDocumentation(propertyValue, metaDBStep);
				}
			}
			if (entry.getKey() != null && entry.getKey().equals("taskVariables")) {
				saveTaskVariable((MapProperty) property, metaDBStep, aSession);
			}

		}

		// AZ - BEGIN -
		// Save Three Additional Property
		// Service Name
		// Service Binding Name
		// Service Type
		Service stepServiceModel = step.getServiceModel();
		ServiceBinding rightServiceBinding = SpagicUtils.getRightStepBinding(step, runtimeBinding);

		String serviceName = "GenericService";
		String serviceEntity = ImConstants.SERVICE_GENERIC;
		String rightServiceBindingName = "AutoGeneratedBinding";

		if (stepServiceModel != null) {
			serviceName = stepServiceModel.getServiceName();

			if (stepServiceModel instanceof JoinControl) {
				serviceEntity = ImConstants.SERVICE_JOIN_CONTROL;
			} else if (stepServiceModel instanceof RouterControl) {
				serviceEntity = ImConstants.SERVICE_ROUTER_CONTROL;
			} else if (stepServiceModel instanceof SplitControl) {
				serviceEntity = ImConstants.SERVICE_SPLIT_CONTROL;
			} else if (stepServiceModel instanceof IterationControl) {
				serviceEntity = ImConstants.SERVICE_ITERATION_CONTROL;
			}
		}
		if (rightServiceBinding != null)
			rightServiceBindingName = rightServiceBinding.getName();

		createMetaDBPropertyForStep(aSession, metaDBStep, ImConstants.IM_SERVICE_NAME, serviceName);

		createMetaDBPropertyForStep(aSession, metaDBStep, ImConstants.IM_SERVICE_BINDING_NAME, rightServiceBindingName);

		createMetaDBPropertyForStep(aSession, metaDBStep, ImConstants.SERVICE_ENTITY_PROPERTY_NAME, serviceEntity);

		// AZ - END

		// Store the relevant data associations
		List<ObservableAttribute> observableAttributes = step.getObservableAttributes();
		int ruleCounter = 0;
		for (ObservableAttribute observableAttribute : observableAttributes) {
			ExtractDataRule extractDataRule = observableAttribute.getObservableAttributeExtractRule();
			//runtimeBinding = retrieveBindingForProcess(process);
			if(runtimeBinding.equalsIgnoreCase("runtime.bpel"))
				saveAttributeAndRules(aSession, observableAttribute, extractDataRule);

			ExtractDataRule dataRule = observableAttribute.getObservableAttributeExtractRule();

			// Retrieve the rule by name (should be unique)
			String ruleName = dataRule.getName();
			Rule rule = AuditDBUtils.getRuleByName(aSession, ruleName);

			org.spagic.metadb.model.StepRuleId metaDBStepRuleId = new org.spagic.metadb.model.StepRuleId(metaDBStep
					.getIdStep(), rule.getIdRule());
			org.spagic.metadb.model.StepRule metaDBStepRule = new org.spagic.metadb.model.StepRule();
			metaDBStepRule.setId(metaDBStepRuleId);
			metaDBStepRule.setOrderId(ruleCounter++);
			aSession.save(metaDBStepRule);
		}

		// Saint - Thu Dec 24 14:34:33 2009
		// 
		// Store Orbeon Form relevant data (if any!)
//		EMap<String, Property> props = step.getProperties();
//		Property propType = props.get("FormType");
//		if ( propType != null && "OrbeonForm".equals(propType.getValue()) ) {
//			saveOrbeonRelevantValues(aSession, props, process, step.getName(), metaDBProcess);
//		}
		//  </Saint>
		return metaDBStep;
	}

	// Saint - Thu Dec 24 15:11:23 2009
	// 
	// New, to save relevant values
//	private void saveOrbeonRelevantValues(Session aSession, EMap<String, Property> props, 
//			org.eclipse.stp.im.Process process,	String taskName, 
//			org.spagic.metadb.model.Process metaDBProcess) {
//
//		// Let's build the Orbeon Form definition URL
//		Property propApplication = props.get("FormApplication");
//		String propApplicationValue = (propApplication==null) ? "" : (String)propApplication.getValue();
//
//		Property propForm = props.get("FormForm");
//		String propFormValue = (propForm==null) ? "" : (String)propForm.getValue();
//
//		String orbeonBaseURL = SpagicUtilsActivator.getDefault()
//		.getPreferenceStore()
//		.getString(SpagicPreferenceConstants.P_WF_SPAGIC_ORBEON_FORM_REPO_URL);
//
//		// creating the URL text
//		StringBuilder formDefUrlText = new StringBuilder(orbeonBaseURL);
//		formDefUrlText.append(propApplicationValue);
//		formDefUrlText.append("/");
//		formDefUrlText.append(propFormValue);
//		formDefUrlText.append("/form/form.xhtml");
//
//		String formDef = "";
//
//		// now we get the form definition source
//		try {
//			InputStream is = new URL(formDefUrlText.toString()).openStream();
//			byte buf[] = new byte[512];
//
//			// download of the form source
//			ByteArrayOutputStream baos = new ByteArrayOutputStream();
//			int gotten=0;
//			while( (gotten = is.read(buf)) > -1) {
//				baos.write(buf, 0, gotten);
//			}
//			formDef = baos.toString();
//			baos.close();
//			baos = null;
//
//		}
//		catch (IOException ex) {
//			throw new OrbeonComunicationIOException(ex.getMessage());
//		}
//
//		// extraction of the relevant data field from the source;
//
//		Matcher relevantValueHunterMatcher = 
//			relevantValueHunterPattern.matcher(formDef);
//
//		// recycling formDefUrlText
//		StringBuilder sb = formDefUrlText;
//		formDefUrlText = null;
//
//		String varValue = "java.lang.String()";
//
//		// Hunt and save the relevant value definition!
//		while (relevantValueHunterMatcher.find()) {
//			Matcher relevantValueNameExtractorMatcher = 
//				relevantValueNameExtractorPattern.matcher(
//						relevantValueHunterMatcher.group(0));
//
//
//			if(relevantValueNameExtractorMatcher.find()) {
//				sb.delete(0, sb.length());
//
//				// fields can be in any section.
//				String varName = relevantValueNameExtractorMatcher.group(1);
//
//				sb.append("_[Task]_");
//				sb.append(taskName);
//				sb.append(".");
//				sb.append(varName);
//
//				org.spagic.metadb.model.Property metaDBProperty = 
//					new org.spagic.metadb.model.Property();
//
//				String sbs = sb.toString();
//
//				metaDBProperty.setCode(sbs);
//				metaDBProperty.setValue(varValue);
//				aSession.save(metaDBProperty);
//
//				// Associate Property and Process
//				PropertyProcessId propertyProcessId =
//					new PropertyProcessId(metaDBProperty.getIdProperty(), 
//							metaDBProcess.getIdProcess());
//
//				PropertyProcess metaDBPropertyProcess = 
//					new PropertyProcess(propertyProcessId, metaDBProcess, 
//							metaDBProperty);
//
//				aSession.save(metaDBPropertyProcess);
//
//				//Create Variable Definition
//
//				org.spagic.metadb.model.VariableDefinition definition = 
//					new org.spagic.metadb.model.VariableDefinition(metaDBProcess, 
//							sbs);
//
//				//			BasicProperty varSchemaBasicProperty = 
//				//				(BasicProperty)process.getProperties().get("Schema[" + varName + "]");
//				//
//				//			if ( varSchemaBasicProperty != null)
//				//				definition.setVarSchema(varSchemaBasicProperty.getValue());
//
//				aSession.save(definition);
//
//
//			}
//
//		}
//
//	}
//  </Saint>

	public void saveAttributeAndRules(Session aSession, ObservableAttribute attribute, ExtractDataRule rule){
		Attribute anAttribute = new Attribute();
		anAttribute.setName(attribute.getName());
		Rule aRule = new Rule();
		aRule.setName(rule.getName());
		final Integer USAGE_EXTRACT_DATA = 0;
		aRule.setRuleUsage(USAGE_EXTRACT_DATA);
		aRule.setExpr(rule.getExpression());
		aRule.setVariableName(rule.getVariableName());
		Attribute attrFromDB = AuditDBUtils.getAttributeByName(aSession, attribute.getName());
		if (attrFromDB == null){
			anAttribute = AuditDBUtils.storeAttribute(aSession, anAttribute);
			aRule.setAttribute(anAttribute);
		} else
			aRule.setAttribute(attrFromDB);
		aRule = AuditDBUtils.storeRule(aSession, aRule);
		rule.setIdRule(Long.toString(aRule.getIdRule()));
	}

	private void saveTaskVariable(MapProperty property, org.spagic.metadb.model.Step metaDBStep, Session session) {
		EMap<String, Property> emap = property.getValue();
		Set<String> keys = emap.keySet();
		for (String key : keys) {
			org.spagic.metadb.model.Property prop = new org.spagic.metadb.model.Property();
			prop.setCode("TaskVar[" + key + "]");



			//TODO mettere a posto!!!
			prop.setValue("java.lang.String");
			session.save(prop);
			PropertyStepId propertyStepId = new PropertyStepId(metaDBStep.getIdStep(), prop.getIdProperty());
			PropertyStep propStep = new PropertyStep(propertyStepId, metaDBStep, prop);
			session.save(propStep);
		}
	}

	private void saveStepDocumentation(String propertyValue, org.spagic.metadb.model.Step metaDBStep) {
		String documentationAbsolutePath = null;
		try {
			documentationAbsolutePath = imFile.getWorkspace().getRoot().getRawLocationURI()
			+ imFile.getProject().getFullPath().toString() + "/" + propertyValue;
			File file = new File(new java.net.URI(documentationAbsolutePath));
			InputStream is = new FileInputStream(file);
			long length = file.length();
			if (length > Integer.MAX_VALUE) {
				System.out.println("File: " + documentationAbsolutePath + " is too large");
			}
			byte[] bytes = new byte[(int) length];
			int offset = 0;
			int numRead = 0;
			while (offset < bytes.length && (numRead = is.read(bytes, offset, bytes.length - offset)) >= 0) {
				offset += numRead;
			}
			if (offset < bytes.length) {
				throw new IOException("Could not completely read file " + file.getName());
			}
			is.close();
			metaDBStep.setDocumentationFileName(file.getName());
			metaDBStep.setDocumentationMimeType(new MimetypesFileTypeMap().getContentType(file));
			metaDBStep.setDocumentation(bytes);
		} catch (Exception e) {
			String message = "Error reading Step Documentation File: " + documentationAbsolutePath;
			SpagicLogger.error(message, e);
			throw new RuntimeException(message, e);
		}
	}

	/**
	 * Saves a single transition
	 * 
	 * @param aSession
	 *          Hibernate session
	 * @param transition
	 *          The transition to save
	 * @param process
	 *          The transition owner process
	 * @param metaDBProcess
	 *          The already saved owner process
	 * @param stepMap
	 *          A map containing all already saved steps
	 * @return The saved transition
	 */
	private org.spagic.metadb.model.Transition saveTransition(Session aSession, org.eclipse.stp.im.Transition transition,
			org.eclipse.stp.im.Process process, org.spagic.metadb.model.Process metaDBProcess,
			Map<String, org.spagic.metadb.model.Step> stepMap) {
		Step sourceStep = transition.getSource();
		Step targetStep = transition.getTarget();

		// Retrieve metaDB steps
		QName sourceStepQName = SpagicUtils.getQNameForStep(process, sourceStep);
		QName targetStepQName = SpagicUtils.getQNameForStep(process, targetStep);

		org.spagic.metadb.model.Step metaDBSourceStep = stepMap.get(sourceStepQName.toString());
		org.spagic.metadb.model.Step metaDBTargetStep = stepMap.get(targetStepQName.toString());

		org.spagic.metadb.model.Transition metaDBTransition = new org.spagic.metadb.model.Transition();
		metaDBTransition.setProcess(metaDBProcess);

		TransitionId transitionId = new TransitionId(metaDBSourceStep.getIdStep(), metaDBTargetStep.getIdStep());
		metaDBTransition.setId(transitionId);
		aSession.save(metaDBTransition);

		return metaDBTransition;
	}

	/**
	 * Produce and save the process graph (actually a SVG).
	 * 
	 * @param aSession
	 *          Hibernate session
	 * @param metaDBProcess
	 *          The saved process
	 */
	private void saveProcessGraph(Session aSession, org.eclipse.stp.im.Process process,
			org.spagic.metadb.model.Process metaDBProcess) {

		// Create the graph to store on the metadb
		String processGraph = GraphGenerator.getGraphForProcess(process, getGraphVizExeLocation(), getGraphVizTempFolder());
		
		if (processGraph == null)
			throw new RuntimeException("Error in generatting SVG Graph, Chech the DOT Parameters in Preference Page");
		// Check if there is a graph already saved
		ProcessDiagram processDiagram = AuditDBUtils.getProcessDiagram(aSession, metaDBProcess);
		if (processDiagram != null) {
			// The diagram already exists: update it
			processDiagram.setSvgDiag(processGraph);
		} else {
			// The diagram doesnt exist: create a new diagram on database
			processDiagram = new ProcessDiagram(metaDBProcess.getIdProcess(), metaDBProcess, processGraph);
		}
		aSession.save(processDiagram);
	}

	/**
	 * Save the iter and all the related associations
	 * 
	 * @param aSession
	 *          Hibernate session
	 * @param imModel
	 *          Intermediate model containing the iter
	 */
	private void saveIter(Session aSession, StpIntermediateModel imModel,
			List<org.spagic.metadb.model.Process> metaDBProcessList) {
		EMap<String, Property> imProperties = imModel.getProperties();
		Property iterNameProperty = null;
		String iterName = null;
		Property iterVersionProperty = null;
		String iterVersion = null;
		org.spagic.metadb.model.Rule ruleForIterTypeDetermination = null;

		// Retrieve iter informations
		iterNameProperty = imProperties.get(ImConstants.IM_ITER_NAME);
		if (iterNameProperty == null) {
			// No iter exists
			return;
		}
		iterName = (String) iterNameProperty.getValue();

		iterVersionProperty = imProperties.get(ImConstants.ITER_VERSION);
		if (iterVersionProperty == null) {
			// No version exists
			return;
		}
		iterVersion = (String) iterVersionProperty.getValue();

		// Create rule for iter determination, if any
		// The rule can also non-exists, if the process belongs to a single iter
		Property ruleForDeterminationProperty = imProperties.get(ImConstants.IM_ITER_RULE_FOR_DETERMINATION);
		if (ruleForDeterminationProperty != null) {
			String ruleForDetermination = (String) ruleForDeterminationProperty.getValue();

			ruleForIterTypeDetermination = new org.spagic.metadb.model.Rule();
			ruleForIterTypeDetermination.setName(RULE_DESCRIPTION + iterName);
			ruleForIterTypeDetermination.setExpr(ruleForDetermination);
			ruleForIterTypeDetermination.setRuleUsage(USAGE_ITER_TYPE_DETERMINATION);
			aSession.save(ruleForIterTypeDetermination);
		}

		// Create Iter
		org.spagic.metadb.model.PoolType poolType = new org.spagic.metadb.model.PoolType(iterName, ITER_DESCRIPTION,
				iterVersion);
		poolType.setRule(ruleForIterTypeDetermination);
		aSession.save(poolType);

		// Retrieve iter attributes
		Property attributeSizeProperty = imProperties.get(ImConstants.IM_ITER_OBSERVABLE_ATTRIBUTE_LIST_SIZE);
		Integer attributeSize = Integer.valueOf((String) attributeSizeProperty.getValue());

		MapProperty mp = (MapProperty) imProperties.get(ImConstants.IM_ITER_OBSERVABLE_ATTRIBUTE_MAP);
		EMap<String, Property> mapValues = mp.getValue();

		for (int i = 0; i < attributeSize; i++) {
			Property bp = mapValues.get(MessageFormat.format(ImConstants.IM_ITER_OBSERVABLE_ATTRIBUTE_NAME, i));

			// The attribute name should be unique
			String attributeName = (String) bp.getValue();
			org.spagic.metadb.model.Attribute attr = AuditDBUtils.getAttributeByName(aSession, attributeName);

			// Save relation attribute-iter
			org.spagic.metadb.model.AttributePool ap = new org.spagic.metadb.model.AttributePool();
			ap.setId(new org.spagic.metadb.model.AttributePoolId(attr.getIdAttribute(), poolType.getIdPoolType()));
			aSession.save(ap);
		}

		// Saves the processes belonging to the iter

		// Retrieve first iter process name
		Property firstProcessNameProperty = imProperties.get(ImConstants.IM_ITER_FIRST_PROCESS_NAME);
		ProcessCollection processCollection = imModel.getProcessCollection();
		List<org.eclipse.stp.im.Process> processList = processCollection.getProcesses();
		int orderCriteria = 1; // This counter increment only if it's not the first
		// process
		int counter = 0; // This counter increment always
		for (org.eclipse.stp.im.Process process : processList) {

			org.spagic.metadb.model.PoolTypeProcess poolTypeProcess = new org.spagic.metadb.model.PoolTypeProcess();
			poolTypeProcess.setId(new org.spagic.metadb.model.PoolTypeProcessId(metaDBProcessList.get(counter++)
					.getIdProcess(), poolType.getIdPoolType()));
			poolTypeProcess.setPoolType(poolType);
			poolTypeProcess.setOrderCriteria(process.getName().equals(firstProcessNameProperty) ? 0 : orderCriteria++);
			aSession.save(poolTypeProcess);
		}
	}

}
