/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.bpel.deploy.ui.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axis2.AxisFault;
import org.apache.ode.axis2.service.ServiceClientUtil;
import org.apache.ode.utils.Namespaces;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.spagic3.utils.SpagicLogger;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;

public class ODEUndeployWorkspaceOperation extends WorkspaceModifyOperation {
	private IFile bpelFile = null;
	
	public ODEUndeployWorkspaceOperation(IFile bpelFile){
		this.bpelFile = bpelFile;
	}
	@Override
	protected void execute(IProgressMonitor arg0) throws CoreException,
			InvocationTargetException, InterruptedException {
	
		//call the webservice to ODE deploy
		String bpelFileName = bpelFile.getName();
		ServiceClientUtil client = new ServiceClientUtil();
		try{
	       OMFactory _factory = OMAbstractFactory.getOMFactory();
	       OMNamespace depns = _factory.createOMNamespace(Namespaces.ODE_DEPLOYAPI_NS, "deployapi");
	       OMElement rootUndeploy = null;
	       
	       String filterString = "name=" + bpelFileName.substring(0, bpelFileName.indexOf(".bpel"));
		   OMElement root = client.buildMessage("listProcesses", new String[] {"filter", "orderKeys"}, new String[] {filterString, ""});
		   OMElement listProcesses = client.send(root, getProcessManagementUrl());
			
			Iterator it = listProcesses.getFirstElement().getChildElements();
			while (it.hasNext()) {
				OMElement process = (OMElement)it.next();
				OMElement packageElement = process.getFirstChildWithName(new QName(Namespaces.ODE_PMAPI_TYPES_NS, "deployment-info")).getFirstChildWithName(new QName(Namespaces.ODE_PMAPI_TYPES_NS, "package"));
				String packageName = packageElement.getText();
				OMElement namePart = _factory.createOMElement("packageName", null);
				namePart.setText(packageName);
				rootUndeploy = _factory.createOMElement("undeploy", depns);
				rootUndeploy.addChild(namePart);
			
			client.send(rootUndeploy, getDeploymentServiceUrl());
			}
       } catch(AxisFault ex) {
       		String msgError = "Problems during deploy." + ex.getMessage();
			SpagicLogger.error(msgError);
			throw new RuntimeException(msgError);
       }
	}
	
	public String getDeploymentServiceUrl() {
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		return prefs.getString(SpagicPreferenceConstants.P_SPAGIC_ODE_URL_DEPLOY);
	}	
	
	public String getProcessManagementUrl() {
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		return prefs.getString(SpagicPreferenceConstants.P_SPAGIC_ODE_URL_PROCESS_MANAGEMENT);
	}	

}
