/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.deploymentservice;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.stp.bpmn.Activity;
import org.eclipse.stp.bpmn.ActivityType;
import org.eclipse.stp.bpmn.BpmnDiagram;
import org.eclipse.stp.bpmn.Pool;
import org.eclipse.stp.bpmn.SubProcess;
import org.eclipse.stp.bpmn.Vertex;
import org.eclipse.stp.im.constants.ImConstants;
import org.spagic3.ui.deploymentservice.wizards.ServiceInfo;
import org.spagic3.ui.deploymentservice.wizards.ServiceInfoRepository;

public class BpmnResolver {

	private List<URI> bpmnURIs;
	private ServiceInfoRepository infoRepository;
	private List<URI> usedBpmnURIs;
	
	public BpmnResolver(List<URI> bpmnURIs, ServiceInfoRepository infoRepository) {
		this.bpmnURIs = bpmnURIs;
		this.infoRepository = infoRepository;
	}
	
	public List<URI> getBpmnURIs() {
		return bpmnURIs;
	}

	public List<URI> getUsedBpmnURIs() {
		return usedBpmnURIs;
	}

	protected BpmnDiagram getBpmnDiagramFromURI(URI bpmnURI){
		ResourceSet bpmnResourceSet = new ResourceSetImpl();
		final Resource bpmnResource = bpmnResourceSet.getResource(org.eclipse.emf.common.util.URI.createURI(bpmnURI.toString()), 
				true);	
		final BpmnDiagram bpmnDiagram = (BpmnDiagram) bpmnResource.getContents().get(0);
		return bpmnDiagram;
		
	}
	
	
	protected boolean considerPool(String poolName, List<String> processIds){
		return  (poolName != null  && processIds.contains(poolName));
	}
	
	public URI getServiceURIForTask(Activity activity){
		final EAnnotation annotation = activity.getEAnnotation(ImConstants.IM_ANNOTATION);
		if (annotation != null) {
			final EMap<String,String> details = annotation.getDetails();
			if (details != null) {
				final String taskType = annotation.getDetails().get(ImConstants.IM_SERVICE_BINDING_NAME);
				final String serviceType = annotation.getDetails().get("ServiceType");
				final String serviceId = annotation.getDetails().get("Service");
				if ("AutomaticTask".equals(taskType)
				 && "OSGi".equals(serviceType)
				 && serviceId != null && !"".equals(serviceId)) {
					final ServiceInfo serviceInfo
							= infoRepository.findServiceInfoById(serviceId);
					if (serviceInfo != null) {
						final URI serviceURI = serviceInfo.getURI();
						return serviceURI;
					}
				}
			}
		}
		return null;
	}
	
	
	public List<URI> getServicesURIForVertexs(EList<Vertex> vertexs) {
		List<URI> uris = new ArrayList<URI>();
		for (Vertex vertex : vertexs) {
			if (vertex instanceof Activity) {
				final Activity activity = (Activity) vertex;
				ActivityType at = activity.getActivityType();

				switch (at.getValue()) {
				case ActivityType.TASK:
					URI serviceURI = getServiceURIForTask(activity);
					if (serviceURI != null) {
						uris.add(getServiceURIForTask(activity));
					}
					break;
				case ActivityType.SUB_PROCESS:
					SubProcess subA = (SubProcess) activity;
					uris.addAll(getServicesURIForVertexs(subA.getVertices()));
					break;
				}
			}
		}
		return uris;
	}
			
	public List<URI> getServiceURIForPool(Pool pool){
		return getServicesURIForVertexs(pool.getVertices());
	}
			
	public List<URI> getServiceURIForBpmn(URI bpmnURI, List<String> processIds){
		List<URI> toBeReturnedURIs = new ArrayList<URI>();
		final BpmnDiagram bpmnDiagram = getBpmnDiagramFromURI(bpmnURI);
		
		for (Pool pool : bpmnDiagram.getPools()) {
			final String poolName = pool.getName();
			if (considerPool(poolName, processIds)){
				usedBpmnURIs.add(bpmnURI);
				toBeReturnedURIs.addAll(getServiceURIForPool(pool));
			}
		}
		return toBeReturnedURIs;
		
	}
	
	
	
	public List<URI> getServiceURIs(List<String> processIds) {
		List<URI> toBeReturnedURIs = new ArrayList<URI>();
		
		usedBpmnURIs = new ArrayList<URI>();
		for (URI bpmnURI : bpmnURIs) {
			toBeReturnedURIs.addAll(getServiceURIForBpmn(bpmnURI, processIds));
		}
		return toBeReturnedURIs;
	}
}
