/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.deploymentservice.actions;

import java.io.ByteArrayInputStream;
import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.stp.bpmn.BpmnDiagram;
import org.eclipse.stp.bpmn.Pool;
import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ImPackage;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.stp.im.in.bpmn2im.Bpmn2ImWorkspaceOperation;
import org.eclipse.stp.im.in.bpmn2im.FixBpmnDiagramAction;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.spagic3.im.out.jbpm.actions.Im2JbpmWorkspaceOperation;
import org.spagic3.metadata.actions.DeployProcessSchemaWorkspaceOperation;
import org.spagic3.ui.deploymentservice.helpers.JPDLValidator;
import org.spagic3.utils.SpagicJBPMPreprocessor;
import org.spagic3.utils.SpagicLogger;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.graph.GraphGenerator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;

public class PreparePackagingArtifacts {

	private IFile bpmnFile = null;
	private List<String> selectedProcessNames;
	private boolean errors = false;
	private boolean generateAll = false;
	
	public boolean isGenerateAll() {
		return generateAll;
	}

	public void setGenerateAll(boolean generateAll) {
		this.generateAll = generateAll;
	}

	/**
	 * Constructor for Action1.
	 */
	public PreparePackagingArtifacts() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	public List<Pool> getBpmnPools(){
		// Read the bpmn emf model
		ResourceSet resourceSetBpmn = new ResourceSetImpl();
	
		org.eclipse.emf.common.util.URI uri = org.eclipse.emf.common.util.URI.createPlatformResourceURI(bpmnFile.getFullPath().toString(),true);
		Resource resourceBpmn = resourceSetBpmn.getResource(uri, true);	
		BpmnDiagram bpmnDiagram= (BpmnDiagram)resourceBpmn.getContents().get(0);
		
		return bpmnDiagram.getPools();
	
	}
	
	public void selectedProcessNames(List<String> selectedProcessNames){
		this.selectedProcessNames = selectedProcessNames;
	}
	
	
	public void doPackaging(List<URI> processRelatedURIs) {
		
		
		Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
	
		
		List<URI> toAdd = new ArrayList<URI>();
		try {
			
			List<Pool> pools = getBpmnPools();
			
			
			if (pools.size() == 0) {
				MessageDialog.openWarning(shell,
						" Error ",
						" The bpmn file ["+bpmnFile.getName()+"] do not contain any pool!!!");
				return;
			}
			
			
			
			
			/**/
			List<String> idSelectedPools = new ArrayList<String>();
			for (Pool p : pools){
				if (generateAll)
					idSelectedPools.add(p.getID());
				else {
					if ( this.selectedProcessNames.contains(p.getName()))
						idSelectedPools.add(p.getID());
				}
			}
			

			WorkspaceModifyOperation operationFixBpmnDiagram = new FixBpmnDiagramAction(bpmnFile, idSelectedPools);
			PlatformUI.getWorkbench().getProgressService().run(false, false, operationFixBpmnDiagram);
			
			
			toAdd.add(bpmnFile.getLocationURI());
			
			
			WorkspaceModifyOperation createIMOperation = new Bpmn2ImWorkspaceOperation(bpmnFile, idSelectedPools);
			((Bpmn2ImWorkspaceOperation)createIMOperation).setSeparateImForEachPool(true);			
			
			PlatformUI.getWorkbench().getProgressService().run(false, false, createIMOperation);
			
			List<IFile> imFiles = ((Bpmn2ImWorkspaceOperation)createIMOperation).getGeneratedFiles();
			
			for (IFile imFile : imFiles){
				packageImFile(imFile, toAdd);
			}
			
		} catch (InvocationTargetException x) {
			errors = true;
			reportError(shell, x);
		} catch (InterruptedException x) {
			errors = true;
			reportError(shell, x);
		} catch (Throwable e) {
			errors = true;
			reportError(shell, e);
		}finally{
			if (!errors){
				processRelatedURIs.addAll(toAdd);
			}
		}
	}
	
	/**
	 * Starting from an IM file, it generates:
	 * 	- The JPDL file
	 * 	- The Graph file
	 * and add them to the list of files to be deployed.
	 * @param imFile
	 * @throws Exception 
	 */
	public void packageImFile(IFile imFile, List<URI> toAdd) throws Exception {		
		SpagicJBPMPreprocessor jbpmProcessor = new SpagicJBPMPreprocessor("runtime.jbpm");
		StpIntermediateModel im = retrieveIMModel(imFile);
		jbpmProcessor.process(imFile, im);
		IFile proxImFile = storeIMPreprocessedFile(imFile, im, null, null);
		
		WorkspaceModifyOperation createJBPMOperation = new Im2JbpmWorkspaceOperation(imFile);
		PlatformUI.getWorkbench().getProgressService().run(false, false, createJBPMOperation);
		
		IFile jpdlFile = getJPDLFileForIMFile(imFile);
		
		JPDLValidator validator = new JPDLValidator(jpdlFile);
		validator.validate();
		if (validator.isErrors())
			throw new RuntimeException("Error Validating jpdl file ["+jpdlFile.getName()+"]\n"+ validator.getErrorMessages().toString());
		IFile graphFile = createGraphFileForIMFile(imFile);
	
		toAdd.add(imFile.getLocationURI());
		toAdd.add(proxImFile.getLocationURI());
		toAdd.add(jpdlFile.getLocationURI());
		toAdd.add(graphFile.getLocationURI());
		
		DeployProcessSchemaWorkspaceOperation prepareProcessSchemaWorkspaceOperation = new DeployProcessSchemaWorkspaceOperation(imFile);
		PlatformUI.getWorkbench().getProgressService().run(false, false, prepareProcessSchemaWorkspaceOperation);
		
		toAdd.addAll(prepareProcessSchemaWorkspaceOperation.getSchemaFiles());
	}

	public boolean isErrors() {
		return errors;
	}

	public void setErrors(boolean errors) {
		this.errors = errors;
	}

	public IFile getImFileForBPMNFile(IFile bpmnFile){
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		String projectName = bpmnFile.getProject().getName();
		IProject proj = root.getProject(projectName);
		String bpmnFileNameRelativeToProject = bpmnFile.getProjectRelativePath().toPortableString();
//		System.out.println("BPMN File Relative To Project " + bpmnFileNameRelativeToProject);
		String imFileRelativeToProject = bpmnFileNameRelativeToProject.replaceFirst("\\.bpmn", ".im");
//		System.out.println("IM File Relative To Project " + imFileRelativeToProject);
		IFile imFile = proj.getFile(imFileRelativeToProject);
		return imFile;
	}
	
	public IFile getGraphFile(IFile imFile){
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		String projectName = imFile.getProject().getName();
		IProject proj = root.getProject(projectName);
		String imFileRelativeToProject = imFile.getProjectRelativePath().toPortableString();
		String graphFileRelativeToProject = imFileRelativeToProject.replaceFirst("\\.im", ".graph");
		
		if (graphFileRelativeToProject.indexOf("IntegrationProcess/") != -1 ){
			graphFileRelativeToProject = graphFileRelativeToProject.replaceFirst("IntegrationProcess/", "Deployables/");
		}else{
			graphFileRelativeToProject = graphFileRelativeToProject.replaceFirst("Processes/", "Deployables/");
		}
		
		
		IFile graphFile = proj.getFile(graphFileRelativeToProject);
		return graphFile;
	}
	
	
	public IFile getImProxFile(IFile imFile){
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		String projectName = imFile.getProject().getName();
		IProject proj = root.getProject(projectName);
		String imFileRelativeToProject = imFile.getProjectRelativePath().toPortableString();
		String imProxFile = imFileRelativeToProject.replaceFirst("\\.im", "_prox.im");
		
		if (imProxFile.indexOf("IntegrationProcess/") != -1 ){
			imProxFile = imProxFile.replaceFirst("IntegrationProcess/", "Deployables/");
		}else{
			imProxFile = imProxFile.replaceFirst("Processes/", "Deployables/");
		}
		
		
		IFile graphFile = proj.getFile(imProxFile);
		return graphFile;
	}
	
	private StpIntermediateModel retrieveIMModel(IFile imFile) {
		// Read the bpmn emf model
		ResourceSet resourceSetIM = new ResourceSetImpl();
		org.eclipse.emf.common.util.URI uri = org.eclipse.emf.common.util.URI.createPlatformResourceURI(imFile.getFullPath().toString(), true);

		// Force Package Registration
		@SuppressWarnings("unused")
		ImPackage imPackage = ImPackage.eINSTANCE;

		// load on demand = true
		Resource resourceIM = resourceSetIM.getResource(uri, true);

		// Get The intermediate model
		StpIntermediateModel imModel = (StpIntermediateModel) resourceIM.getContents().get(0);
		return imModel;
	}
	public IFile createGraphFileForIMFile(IFile imFile){
		StpIntermediateModel im = retrieveIMModel(imFile);
		SpagicJBPMPreprocessor preprocessor = new SpagicJBPMPreprocessor("runtime.jbpm");
		
		preprocessor.process(imFile, im);
		
		
		org.eclipse.stp.im.Process aProcess = null;
		
		for (org.eclipse.stp.im.Process process : im.getProcessCollection().getProcesses()){
			BasicProperty isSubProcess = (BasicProperty) process.getProperties().get(ImConstants.IS_SUB_PROCESS);
			boolean isSubProcessBool  = isSubProcess != null ? Boolean.valueOf(isSubProcess.getValue()) : false;
			if (!isSubProcessBool)
				aProcess = process;
		}
		
		
		//
		// Create the graph to store on the metadb
		String processGraph = GraphGenerator.getGraphForProcess(aProcess, getGraphVizExeLocation(), getGraphVizTempFolder());
		
		if (processGraph == null)
			throw new RuntimeException("Cannot Generate Graph File, Check the DOT Parameters in Preference Page");
		
		IFile graphFile = getGraphFile(imFile);
		ByteArrayInputStream iStream = new ByteArrayInputStream(processGraph.getBytes());
		
		
		try{
			if (graphFile.exists()) {
				graphFile.setContents(iStream, true, true,  null);
			} else {
				graphFile.create(iStream, true, null);
			}
		}catch (Exception e) {
			throw new RuntimeException(e.getMessage(), e);
		}
		
		return graphFile;
		
	}
	
	public IFile getJPDLFileForIMFile(IFile imFile){
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		String projectName = imFile.getProject().getName();
		IProject proj = root.getProject(projectName);
		String imFileRelativeToProject = imFile.getProjectRelativePath().toPortableString();
		String jpdlFileRelativeToProject = imFileRelativeToProject.replaceFirst("\\.im", ".jpdl");
		
		if (jpdlFileRelativeToProject.indexOf("IntegrationProcess/") != -1 ){
			jpdlFileRelativeToProject = jpdlFileRelativeToProject.replaceFirst("IntegrationProcess/", "Deployables/");
		}else{
			jpdlFileRelativeToProject = jpdlFileRelativeToProject.replaceFirst("Processes/", "Deployables/");
		}
		
		
		IFile jpdlFile = proj.getFile(jpdlFileRelativeToProject);
		return jpdlFile;
	}
	

	
	/**
	 * Displays an error that occured during the project creation.
	 * 
	 * @param x
	 *            details on the error
	 */
	private void reportError(Shell shell, Throwable x) {
		SpagicLogger.error(x.getMessage(), x);
		ErrorDialog.openError(shell, "Error", "Error Occurred", makeStatus(x));
	}

	public StringBuffer getCauseFromThrowable(Throwable x){
		StringBuffer causeMessage = new StringBuffer();
		if (x.getMessage() != null)
			causeMessage.append(x.getMessage()+"\n");
		
		if (x.getCause() != null){
			if (x.getCause().getMessage() != null){
				causeMessage.append(x.getCause().getMessage()+"\n");
			}
		}
		return causeMessage;
	}
	public IStatus makeStatus(Throwable x) {

		StringBuffer causeMessage = new StringBuffer();
		
		if (x instanceof InvocationTargetException){
			Throwable target = ((InvocationTargetException)x).getTargetException();
			causeMessage.append(getCauseFromThrowable(target));
		}else{
			causeMessage = getCauseFromThrowable(x);
		}
		
		return new Status(IStatus.ERROR, 
							org.spagic3.ui.deploymentservice.Activator.PLUGIN_ID, 
							IStatus.ERROR,
				causeMessage.toString(), null);

	}
	
	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		
		setBpmnFile((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}
	public IFile getBpmnFile() {
		return bpmnFile;
	}

	public void setBpmnFile(IFile bpmnFile) {
		this.bpmnFile = bpmnFile;
	}

	/**
	 * @return The location of the Graphviz executable.
	 */
	public String getGraphVizExeLocation() {
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		return prefs.getString(SpagicPreferenceConstants.P_GRAPH_VIZ_DOT_EXE);
	}

	/**
	 * @return The location of the temporary folder used for generating graphs.
	 */
	public String getGraphVizTempFolder() {
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		return prefs.getString(SpagicPreferenceConstants.P_GRAPH_VIZ_TEMP_FOLDER);
	}
	
	public IFile storeIMPreprocessedFile(IFile originalIMFile, StpIntermediateModel processedIm, IProgressMonitor monitor, String forcedImFileName) throws Exception {
		
//		IContainer container = originalIMFile.getParent();

		// Create a resource set.
		ResourceSet resourceSetStpIm = new ResourceSetImpl();

		
		IFile imProxFile = getImProxFile(originalIMFile);
		org.eclipse.emf.common.util.URI destinationStpFileURI = org.eclipse.emf.common.util.URI.createPlatformResourceURI(imProxFile.getFullPath().toString(), true);

		// Create a resource for this file.
		Resource resourceStpIm = resourceSetStpIm.createResource(destinationStpFileURI);
		resourceStpIm.getContents().add(processedIm);

		resourceStpIm.save(Collections.EMPTY_MAP);

		
		return imProxFile;

	}
}
