/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.deploymentservice.wizards;

import java.io.FileInputStream;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.URIUtil;
import org.eclipse.ebpm.service.model.ServiceModelHelper;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.spagic3.ui.deploymentservice.Activator;
import org.spagic3.ui.deploymentservice.BpmnResolver;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


public class ProcessDeploymentPage extends WizardPage {

	private Table processTable;
	private CheckboxTableViewer processTableViewer;
	private IProject project;
	private ServiceInfoRepository infoRepository;
	private List<URI> processURIs;
	private List<URI> bpmnURIs;
	private BpmnResolver bpmnResolver;
//	private boolean initialized = false;
	private boolean createZipArchive;
	private Text archiveNameTextField;
	private String archiveName;
	
	public ProcessDeploymentPage(ISelection selection) {
		super("processDeploymentPage");
		setTitle("Deployment Service");
		setDescription("This wizard will help you deploy services");
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 2;
		layout.verticalSpacing = 9;

		Label label = new Label(container, SWT.NULL);
		label.setText("Select processes for deployment");
		createSpacer(container, 1);

		Composite tableComposite = new Composite(container, SWT.NULL);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL | GridData.FILL_VERTICAL);
		gd.horizontalSpan = 1;
		tableComposite.setLayoutData(gd);
		tableComposite.setLayout(new FillLayout());

		processTable = new Table(tableComposite, SWT.CHECK | SWT.V_SCROLL);
		
		processTableViewer = new CheckboxTableViewer(processTable);
		processTableViewer.setLabelProvider(new ProcessLabelProvider());
		processTableViewer.setContentProvider(new ArrayContentProvider());
		processTableViewer.addSelectionChangedListener(
				new ISelectionChangedListener() {
					public void selectionChanged(SelectionChangedEvent e) {
						dialogChanged();
					}
		});

		//process buttons
		Composite processButtons = new Composite(container, SWT.NULL);
		gd = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
		processButtons.setLayoutData(gd);
		processButtons.setLayout(new FillLayout(SWT.VERTICAL));

		Button selectAllButton = new Button(processButtons, SWT.PUSH);
		selectAllButton.setText("Select All");
		selectAllButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {}
			public void widgetSelected(SelectionEvent e) {
				processTableViewer.setCheckedElements(
						((IStructuredContentProvider) processTableViewer
								.getContentProvider()).getElements(processURIs));
				dialogChanged();
			}
		});

		Button selectNoneButton = new Button(processButtons, SWT.PUSH);
		selectNoneButton.setText("Select None");
		selectNoneButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {}
			public void widgetSelected(SelectionEvent e) {
				processTableViewer.setCheckedElements(new Object[0]);
				dialogChanged();
			}
		});

		Button invertSelectionButton = new Button(processButtons, SWT.PUSH);
		invertSelectionButton.setText("Invert Selection");
		invertSelectionButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {}
			public void widgetSelected(SelectionEvent e) {
				List<URI> unselectedURIs = new LinkedList<URI>();
				unselectedURIs.addAll(processURIs);
				unselectedURIs.removeAll(Arrays.asList(processTableViewer.getCheckedElements()));
				processTableViewer.setCheckedElements(unselectedURIs.toArray());
				dialogChanged();
			}
		});
			
		final Button createZipCheckBox = new Button(container, SWT.CHECK);
		createZipCheckBox.setText("Create zip archive");
		createZipCheckBox.setSelection(false);
		gd = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
		createZipCheckBox.setLayoutData(gd);
		createZipCheckBox.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {}
			public void widgetSelected(SelectionEvent e) {
					setCreateZipArchive(createZipCheckBox.getSelection());
					if(createZipCheckBox.getSelection()){
						archiveNameTextField.setVisible(true);
						archiveNameTextField.setFocus();
						MessageDialog.openWarning(getShell(), "Warning deploy procedure disabled", 
								"This option will create a Zip archive under Deployables folder, therefore no files will be deployed into Spagic Home folder.");
					}else{
						archiveNameTextField.setVisible(false);
					}
					dialogChanged();
			}
		});
		
		createSpacer(container, 1);
		
		archiveNameTextField = new Text(container, SWT.BORDER);
		archiveNameTextField.setEditable(true);
		archiveNameTextField.setText("ArchiveName.zip");
		archiveNameTextField.setVisible(false);
		gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		archiveNameTextField.setLayoutData(gd);
		archiveNameTextField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setArchiveName(archiveNameTextField.getText());				
			}
		});
		
				
		initialize();
		dialogChanged();
		setControl(container);
	}
	
	private void createSpacer(Composite parent, int span) {
		Label spacer = new Label(parent, SWT.NULL);
		GridData gd = new GridData();
		gd.horizontalSpan = span;
		spacer.setLayoutData(gd);
	}

	private class ProcessLabelProvider extends LabelProvider {
		public Image getImage(Object element) {
			String extension = null;
			if (element instanceof URI) {
				final String filename = URIUtil.lastSegment((URI) element);
				int dotIndex = filename.lastIndexOf(".");
				extension = dotIndex == -1 ? null : filename.substring(dotIndex + 1);
			}
			if ("connector".equals(extension)) {
				return Activator.getDefault().getImage(Activator.IMG_CONNECTOR);
			}
			if ("service".equals(extension)) {
				return Activator.getDefault().getImage(Activator.IMG_SERVICE);
			}
			return null;
		}
		public String getText(Object element) {
			try {
				if (element instanceof URI) {
					final String filename = URIUtil.lastSegment((URI) element);
					final DocumentBuilderFactory xmlFact = DocumentBuilderFactory.newInstance();
					xmlFact.setNamespaceAware(true);
			        final DocumentBuilder builder = xmlFact.newDocumentBuilder();
			        final Document doc = builder.parse(new FileInputStream(((URI) element).getPath()));
					final String id = ServiceModelHelper.evalXPathAsString(doc, "/spagic:component/@spagic.id");
					return id + " ( " + filename + " )";
				}
			} catch (ParserConfigurationException e) {
				e.printStackTrace();
			} catch (SAXException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
			return null;
		}
	}
	
	private void initialize() {}

	
	public void setVisible(boolean visible) {
		if (visible) {
			
		
			
			IWizardPage previousPage = getWizard().getPreviousPage(this);
//			if (!initialized) {
				if (previousPage instanceof ProcessDeploymentPage) {
					final ProcessDeploymentPage processPage
							= (ProcessDeploymentPage) previousPage;
					setProject(processPage.getProject());
				} else if (previousPage instanceof ServiceDeploymentChoicePage) {
					final ServiceDeploymentChoicePage choicePage
							= (ServiceDeploymentChoicePage) previousPage;
					setProject(choicePage.getProject());
				}
				infoRepository = Activator.getDefault()
						.getServiceInfoRepository(getProject());
				processURIs = infoRepository.getProjectProcessURIs();
				processTableViewer.setInput(processURIs);
				bpmnResolver = new BpmnResolver(
						getBpmnURIs(), 
						infoRepository);
//				initialized = true;
//			}
			dialogChanged();
		}
		super.setVisible(visible);
	}
	
	private void dialogChanged() {
		if (processTableViewer.getCheckedElements().length == 0) {
			updateStatus("Select processes to deploy from the list below");
			return;
		}
		updateStatus(null);
	}

	private void updateStatus(String message) {
		setMessage(message);
		setPageComplete(message == null);
	}

	public void setProject(IProject project) {
		this.project = project;
	}

	public IProject getProject() {
		return project;
	}

	public boolean isCreateZipArchive() {
		return createZipArchive;
	}

	public void setCreateZipArchive(boolean createZipArchive) {
		this.createZipArchive = createZipArchive;
	}	

	public String getArchiveName() {
		return archiveName;
	}

	public void setArchiveName(String archiveName) {
		this.archiveName = archiveName;
	}

	public List<URI> getSelectedURIs() {
		List<URI> toBeReturnedURIs = new ArrayList<URI>();
		for (Object checked : processTableViewer.getCheckedElements()) {
			toBeReturnedURIs.add((URI) checked);
		}
		return toBeReturnedURIs;
	}

	public List<URI> getSelectedURIsWithDependencies() {
		List<URI> toBeReturnedURIs = getSelectedURIs();
		resolveDependencies(toBeReturnedURIs);
		return toBeReturnedURIs;
	}

	public List<URI> getSelectedBpmnURIs() {
		if (bpmnResolver != null){
			return bpmnResolver.getUsedBpmnURIs();
		}else{
			return new ArrayList<URI>();
		}
	}

	private void resolveDependencies(List<URI> selected) {
		List<URI> toBeAddedURIs = new ArrayList<URI>();
		final List<String> processIds = new ArrayList<String>();
		for (URI uri : selected) {
			final ServiceInfo info = infoRepository.findServiceInfoByURI(uri);
			final String processId = info.getProcess();
			if (processId != null && !"".equals(processId)) {
				processIds.add(processId);
				List<ServiceInfo> targeterInfos
						= infoRepository.findServiceInfosByTarget(
								info.getServiceId());
				if (targeterInfos != null) {
					for (ServiceInfo targeterInfo : targeterInfos) {
						final URI targeterURI = targeterInfo.getURI();
						if (targeterURI != null 
								&& !selected.contains(targeterURI)
								&& !toBeAddedURIs.contains(targeterURI)) {
							toBeAddedURIs.add(targeterURI);
						}
					}
				}
				final String targetId = info.getTarget();
				if (targetId != null && !"".equals(targetId)) {
					ServiceInfo targetedInfo
							= infoRepository.findServiceInfoById(targetId);
					if (targetedInfo != null) {
						final URI targetedURI = targetedInfo.getURI();
						if (targetedURI != null 
								&& !selected.contains(targetedURI)
								&& !toBeAddedURIs.contains(targetedURI)) {
							toBeAddedURIs.add(targetedURI);
						}
					} //not found
				}
			}
		}
		if (!toBeAddedURIs.isEmpty()) {
			selected.addAll(toBeAddedURIs);
		}
		
		for (URI serviceURI : bpmnResolver.getServiceURIs(processIds)) {
			if (!selected.contains(serviceURI)) {
				selected.add(serviceURI);
			}
		}
	}
	
	public List<URI> getBpmnURIs() {
		if (bpmnURIs == null) {
			bpmnURIs = Activator.getDefault().getBpmnURIs(getProject());
		}
		return bpmnURIs;
	}

}