/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.deploymentservice.wizards;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.io.SAXReader;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.stp.bpmn.BpmnDiagram;
import org.eclipse.stp.bpmn.Pool;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.spagic3.ui.deploymentservice.Activator;

public class ServiceDeploymentChoicePage extends WizardPage {

	private TableViewer projectsTableViewer;
	private IProject project;
	private Button serviceDeploy;
	private Button processDeploy;
	private boolean isProcessDeploy = false;
	private ISelection selection;
	private boolean showProjectList;
	
	public ServiceDeploymentChoicePage(ISelection selection, boolean showProjectList) {
		super("serviceDeploymentChoice");
		setTitle("Deployment Service");
		setDescription("This wizard will help you deploy services");
		this.selection = selection;
		this.showProjectList = showProjectList;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 1;
		layout.verticalSpacing = 9;
		
		if (showProjectList) {
			Label listLabel = new Label(container, SWT.NULL);
			listLabel.setText("Select a project");
	
			projectsTableViewer = new TableViewer(container, SWT.SINGLE | SWT.V_SCROLL);
			GridData gd = new GridData(GridData.FILL_BOTH);
			projectsTableViewer.getTable().setLayoutData(gd);
			projectsTableViewer.setLabelProvider(
					new ProjectLabelProvider());
			projectsTableViewer.setContentProvider(
					new ArrayContentProvider());
			projectsTableViewer.setInput(
					ResourcesPlugin.getWorkspace().getRoot().getProjects());
			projectsTableViewer.addSelectionChangedListener(
					new ISelectionChangedListener() {
						public void selectionChanged(SelectionChangedEvent e) {
							final ISelection selection = e.getSelection();
							if (selection != null) {
								Object selected
										= ((IStructuredSelection) selection)
												.getFirstElement();
								setProject((IProject) selected);
							}
							dialogChanged();
						}
			});
		}
		
		Label choiceLabel = new Label(container, SWT.NULL);
		choiceLabel.setText("Choose a deployment mode");

		serviceDeploy = new Button(container, SWT.RADIO);
		serviceDeploy.setText("Deploy services");
		GridData gd = new GridData();
		serviceDeploy.setLayoutData(gd);
		serviceDeploy.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0) {
				setProcessDeploy(false);
			}
		});

		processDeploy = new Button(container, SWT.RADIO);
		processDeploy.setText("Deploy processes and related services");
		gd = new GridData();
		processDeploy.setLayoutData(gd);
		processDeploy.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent arg0) {}
			public void widgetSelected(SelectionEvent arg0) {
				setProcessDeploy(true);
			}
		});
		processDeploy.setSelection(true);
		setProcessDeploy(true);
		initialize();
		dialogChanged();
		setControl(container);
	}
	
	private class ProjectLabelProvider extends LabelProvider {
		public Image getImage(Object element) {
			return Activator.getDefault().getImage(Activator.IMG_PROJECTS);
		}
		public String getText(Object element) {
			if (element instanceof IProject) {
				final IProject project = (IProject) element;
				return project.getName();
			}
			return null;
		}
	}

	private void initialize() {
		if (selection != null && selection.isEmpty() == false
				&& selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1)
				return;
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				if (showProjectList) {
					projectsTableViewer.setSelection(
							new StructuredSelection(
									((IResource) obj).getProject()));
				} else {
					setProject(((IResource) obj).getProject());
				}
			}
		}
	}

	private void dialogChanged() {
		if (showProjectList 
				&& projectsTableViewer.getSelection().isEmpty()) {
			updateStatus("Select a project for deployment");
			return;
		}
		updateStatus(null);
	}

	private void updateStatus(String message) {
		setMessage(message);
		setPageComplete(message == null);
		
	}

	
	
	public IProject getProject() {
		return project;
	}

	public void setProject(IProject project) {
		this.project = project;
		syncBPMNProcessesWithOrchestrationServices();
	}

	public boolean isProcessDeploy() {
		return isProcessDeploy;
	}

	public void setProcessDeploy(boolean isProcessDeploy) {
		this.isProcessDeploy = isProcessDeploy;
	}
	
	public List<Pool> getBpmnPools(IFile bpmnFile){
		// Read the bpmn emf model
		ResourceSet resourceSetBpmn = new ResourceSetImpl();
		org.eclipse.emf.common.util.URI uri = org.eclipse.emf.common.util.URI.createPlatformResourceURI(bpmnFile.getFullPath().toString(),true);
		Resource resourceBpmn = resourceSetBpmn.getResource(uri, true);	
		
		BpmnDiagram bpmnDiagram= (BpmnDiagram)resourceBpmn.getContents().get(0);
		return bpmnDiagram.getPools();
	
	}
	
	public List<IFile> getBPMNFilesFromIProject(IProject project){
		List<IFile> ifiles  = new ArrayList<IFile>();
		
		fillIFilesWithExtension(ifiles, project, "bpmn");
		
		return ifiles;
	}
	
	public List<IFile> getServicesFileFromIproject(IProject project){
		List<IFile> ifiles  = new ArrayList<IFile>();
		
		fillIFilesWithExtension(ifiles, project, "service");
		
		return ifiles;
	}
	
	public void fillIFilesWithExtension(List<IFile> files, IResource resource, String ext){
		if (resource instanceof IFile) {
		final IFile file = (IFile) resource;
		final String extension = file.getFileExtension();
		if (ext.equals(extension)) {
			files.add((IFile)resource);
		}
	} else if (resource instanceof IContainer) {
		final IContainer container = (IContainer) resource;
		try {
			for (IResource member : container.members(true)) {
				fillIFilesWithExtension(files, member, ext);
			}
		} catch (CoreException e) {}
	}
	}
	
	public void syncBPMNProcessesWithOrchestrationServices(){
		
			IProject project = getProject();
			List<IFile> bpmnFiles = getBPMNFilesFromIProject(project);
			
			Map<String,IFile> poolToOrchestrationServiceFile = buildPoolToOrchestrationServiceMap(project);
			
			for (IFile bpmnFile : bpmnFiles){
				String fileName = bpmnFile.getName();
				List<Pool> pools = getBpmnPools(bpmnFile);
				
				for (Pool aPool : pools){
					
					IFile iFile = poolToOrchestrationServiceFile.get(aPool.getName());
					
					if (iFile == null){
						if (MessageDialog.openConfirm(getShell(), "Spagic", "In ["+fileName+"] the Pool ["+aPool.getName()+"] has not associate no BPM Service!! Create It ?"))
							iFile = getOrchestrationServiceIFile(project, aPool.getName());
							if (!iFile.exists())
								createOrchestrationServiceForPool(iFile, aPool.getName());
					}
				}
			}
	}

	public Map<String, IFile> buildPoolToOrchestrationServiceMap(IProject project){
		Map<String, IFile> orchestrationServiceMap = new HashMap<String, IFile>();
		ServiceInfoRepository repo = Activator.getDefault().getServiceInfoRepository(project);
		
		List<IFile> serviceFiles = getServicesFileFromIproject(project);
		
		for (IFile serviceFile : serviceFiles){
			ServiceInfo sInfo = repo.findServiceInfoByFile(serviceFile);
			String processName = sInfo.getProcess();
			if ( processName != null)
				orchestrationServiceMap.put(processName, serviceFile);
		}
		return orchestrationServiceMap;
		
	}
	
	public Document getDocumentFromIFile(IFile ifile){
		try{
			SAXReader reader = new SAXReader();
			return reader.read(ifile.getContents());
		}catch (Exception e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}
	
	public IFile getOrchestrationServiceIFile(IProject proj, String poolName){
		IContainer container = proj.getFolder("Services");
		String bpmGatewayServiceID =  poolName + "Orchestration";
		String bpmGatewayServiceFileName = bpmGatewayServiceID+".service";
		final IFile file = container.getFile(new Path(bpmGatewayServiceFileName));
		return file;
	}
	
	public void createOrchestrationServiceForPool(IFile file, String poolName){
		String bpmGatewayServiceID =  poolName + "Orchestration";
		String bpmGatewayServiceFileName = bpmGatewayServiceID+".service";
		try {
			InputStream stream = openContentStreamForBPMGateway(bpmGatewayServiceID,bpmGatewayServiceFileName,poolName);
			file.create(stream, true,  null);
			stream.close();
		}catch (Exception e) {
			MessageDialog.openError(getShell(), "Error", e
					.getMessage());
		}

	}
	
	 private static InputStream openContentStreamForBPMGateway(String bpmGatewayServiceId,String fileName, String poolName) {
			StringBuffer xml = new StringBuffer("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			xml.append("<spagic:component\n");
			xml.append("\t\txmlns:spagic=\"urn:org:spagic3\"\n");
			xml.append("\t\txmlns=\"urn:org:spagic3\"\n");
			xml.append("\t\tspagic.id=\""+bpmGatewayServiceId+"\"\n");
			xml.append("\t\tfactory.name=\"spagic3.bpmfactory\">\n");

		
			xml.append("\t<property name=\"")
						.append("process")
						.append("\" value=\"")
						.append(poolName)
						.append("\"/>\n");
			
			xml.append("</spagic:component>");
			return new ByteArrayInputStream(xml.toString().getBytes());
		}

}