/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.deploymentservice.wizards;

import java.io.FileInputStream;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.URIUtil;
import org.eclipse.ebpm.service.model.ServiceModelHelper;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.spagic3.ui.deploymentservice.Activator;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


public class ServiceDeploymentPage extends WizardPage {

	private Table serviceTable;
	private CheckboxTableViewer serviceTableViewer;
	private IProject project;
	private ServiceInfoRepository infoRepository;
	private List<URI> serviceURIs;
//	private boolean initialized = false;
	private boolean createZipArchive = false;
	private Text archiveNameTextField;
	private Button createZipCheckBox;
	private String archiveName;

	public ServiceDeploymentPage(ISelection selection) {
		super("serviceDeploymentPage");
		setTitle("Deployment Service");
		setDescription("This wizard will help you deploy services");
	}
	
	public boolean isCreateZipArchive() {
		return createZipArchive;
	}

	public void setCreateZipArchive(boolean createZipArchive) {
		this.createZipArchive = createZipArchive;
	}	

	public String getArchiveName() {
		return archiveName;
	}

	public void setArchiveName(String archiveName) {
		this.archiveName = archiveName;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 2;
		layout.verticalSpacing = 9;

		Label label = new Label(container, SWT.NULL);
		label.setText("Select services for deployment");
		createSpacer(container, 1);

		Composite tableComposite = new Composite(container, SWT.NULL);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL | GridData.FILL_VERTICAL);
		gd.horizontalSpan = 1;
		tableComposite.setLayoutData(gd);
		tableComposite.setLayout(new FillLayout());

		serviceTable = new Table(tableComposite, SWT.CHECK | SWT.V_SCROLL);
		
		serviceTableViewer = new CheckboxTableViewer(serviceTable);
		serviceTableViewer.setLabelProvider(new ServiceLabelProvider());
		serviceTableViewer.setContentProvider(new ArrayContentProvider());
		serviceTableViewer.addSelectionChangedListener(
				new ISelectionChangedListener() {
					public void selectionChanged(SelectionChangedEvent e) {
						dialogChanged();
					}
		});

		//service buttons
		Composite serviceButtons = new Composite(container, SWT.NULL);
		gd = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
		serviceButtons.setLayoutData(gd);
		serviceButtons.setLayout(new FillLayout(SWT.VERTICAL));

		Button selectAllButton = new Button(serviceButtons, SWT.PUSH);
		selectAllButton.setText("Select All");
		selectAllButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {}
			public void widgetSelected(SelectionEvent e) {
				serviceTableViewer.setCheckedElements(
						((IStructuredContentProvider) serviceTableViewer
								.getContentProvider()).getElements(serviceURIs));
				dialogChanged();
			}
		});

		Button selectNoneButton = new Button(serviceButtons, SWT.PUSH);
		selectNoneButton.setText("Select None");
		selectNoneButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {}
			public void widgetSelected(SelectionEvent e) {
				serviceTableViewer.setCheckedElements(new Object[0]);
				dialogChanged();
			}
		});

		Button invertSelectionButton = new Button(serviceButtons, SWT.PUSH);
		invertSelectionButton.setText("Invert Selection");
		invertSelectionButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {}
			public void widgetSelected(SelectionEvent e) {
				List<URI> unselectedURIs = new LinkedList<URI>();
				unselectedURIs.addAll(serviceURIs);
				unselectedURIs.removeAll(Arrays.asList(serviceTableViewer.getCheckedElements()));
				serviceTableViewer.setCheckedElements(unselectedURIs.toArray());
				dialogChanged();
			}
		});
		
		new Label(serviceButtons, SWT.NULL); //spacer

		Button resolveDependenciesButton = new Button(serviceButtons, SWT.PUSH);
		resolveDependenciesButton.setText("Resolve Dependencies");
		resolveDependenciesButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {}
			public void widgetSelected(SelectionEvent e) {
				List<URI> toBeSelectedURIs = new ArrayList<URI>();
				for (Object checked : serviceTableViewer.getCheckedElements()) {
					toBeSelectedURIs.add((URI) checked);
				}
				resolveDependencies(toBeSelectedURIs);
				serviceTableViewer.setCheckedElements(toBeSelectedURIs.toArray());
				dialogChanged();
			}
		});
		
		createZipCheckBox = new Button(container, SWT.CHECK);
		createZipCheckBox.setText("Create zip archive");
		createZipCheckBox.setSelection(isCreateZipArchive());		
		gd = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
		createZipCheckBox.setLayoutData(gd);
		createZipCheckBox.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {}
			public void widgetSelected(SelectionEvent e) {
					setCreateZipArchive(createZipCheckBox.getSelection());
					if(createZipCheckBox.getSelection()){
						archiveNameTextField.setVisible(true);
						archiveNameTextField.setFocus();
						MessageDialog.openWarning(getShell(), "Warning deploy procedure disabled", 
								"This option will create a Zip archive under Deployables folder, therefore no files will be deployed into Spagic Home folder.");
					}else{
						archiveNameTextField.setVisible(false);
					}
					dialogChanged();
			}
		});
		
		
		gd = new GridData();
		createZipCheckBox.setLayoutData(gd);
		createZipCheckBox.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {}
			public void widgetSelected(SelectionEvent e) {
					setCreateZipArchive(createZipCheckBox.getSelection());
					dialogChanged();
			}
		});
		
		createSpacer(container, 1);
		
		archiveNameTextField = new Text(container, SWT.BORDER);
		archiveNameTextField.setEditable(true);
		archiveNameTextField.setVisible(false);
		archiveNameTextField.setText("ArchiveName.zip");
		gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		archiveNameTextField.setLayoutData(gd);
		archiveNameTextField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setArchiveName(archiveNameTextField.getText());				
			}
		});

		initialize();
		dialogChanged();
		setControl(container);
	}
	
	protected void updateZipFields(){
		createZipCheckBox.setSelection(isCreateZipArchive());			
		if(isCreateZipArchive()){
			createZipCheckBox.setEnabled(false);
			archiveNameTextField.setEditable(false);
			archiveNameTextField.setVisible(true);
			archiveNameTextField.setText(getArchiveName()!=null?getArchiveName():"ArchiveName.zip");
		}else{
			createZipCheckBox.setEnabled(true);
			archiveNameTextField.setEditable(true);
			archiveNameTextField.setVisible(false);
			archiveNameTextField.setText("ArchiveName.zip");
		}
	}
	
	private void resolveDependencies(List<URI> selected) {
		boolean targetAdded;
		List<URI> toBeResolvedURIs = selected;
		List<URI> toBeAddedURIs = new ArrayList<URI>();
		do {
			targetAdded = false;
			for (URI uri : toBeResolvedURIs) {
				final ServiceInfo info = infoRepository.findServiceInfoByURI(uri);
				final String targetId = info.getTarget();
				if (targetId != null && !"".equals(targetId)) {
					final ServiceInfo targetInfo
							= infoRepository.findServiceInfoById(targetId);
					if (targetInfo == null) {
						//not found
						continue;
					}
					final URI targetURI = targetInfo.getURI();
					if (targetURI != null 
							&& !selected.contains(targetURI)
							&& !toBeAddedURIs.contains(targetURI)) {
						toBeAddedURIs.add(targetURI);
						targetAdded = true;
					}
				}
			}
			if (!toBeAddedURIs.isEmpty()) {
				selected.addAll(toBeAddedURIs);
				toBeResolvedURIs = toBeAddedURIs;
				toBeAddedURIs = new ArrayList<URI>();
			}
		} while (targetAdded);
	}

	private void createSpacer(Composite parent, int span) {
		Label spacer = new Label(parent, SWT.NULL);
		GridData gd = new GridData();
		gd.horizontalSpan = span;
		spacer.setLayoutData(gd);
	}

	private class ServiceLabelProvider extends LabelProvider {
		public Image getImage(Object element) {
			String extension = null;
			if (element instanceof URI) {
				final String filename = URIUtil.lastSegment((URI) element);
				int dotIndex = filename.lastIndexOf(".");
				extension = dotIndex == -1 ? null : filename.substring(dotIndex + 1);
			}
			if ("connector".equals(extension)) {
				return Activator.getDefault().getImage(Activator.IMG_CONNECTOR);
			}
			if ("service".equals(extension)) {
				return Activator.getDefault().getImage(Activator.IMG_SERVICE);
			}
			return null;
		}
		public String getText(Object element) {
			try {
				if (element instanceof URI) {
					final String filename = URIUtil.lastSegment((URI) element);
					final DocumentBuilderFactory xmlFact = DocumentBuilderFactory.newInstance();
					xmlFact.setNamespaceAware(true);
			        final DocumentBuilder builder = xmlFact.newDocumentBuilder();
			        final Document doc = builder.parse(new FileInputStream(((URI) element).getPath()));
					final String id = ServiceModelHelper.evalXPathAsString(doc, "/spagic:component/@spagic.id");
					return id + " ( " + filename + " )";
				}
			} catch (ParserConfigurationException e) {
				e.printStackTrace();
			} catch (SAXException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
			return null;
		}
	}
	
	private void initialize() {}
	
	public void setVisible(boolean visible) {
		if (visible) {
			IWizardPage previousPage = getWizard().getPreviousPage(this);
//			if (!initialized) {
				if (previousPage instanceof ProcessDeploymentPage) {
					final ProcessDeploymentPage processPage
							= (ProcessDeploymentPage) previousPage;
					setProject(processPage.getProject());
				} else if (previousPage instanceof ServiceDeploymentChoicePage) {
					final ServiceDeploymentChoicePage choicePage
							= (ServiceDeploymentChoicePage) previousPage;
					setProject(choicePage.getProject());
				}
				infoRepository = Activator.getDefault()
						.getServiceInfoRepository(getProject());
				serviceURIs = infoRepository.getProjectServiceURIs();
				serviceTableViewer.setInput(serviceURIs);
//				initialized = true;
//			}
			if (previousPage instanceof ProcessDeploymentPage) {
				final ProcessDeploymentPage processPage
						= (ProcessDeploymentPage) previousPage;
				serviceTableViewer.setCheckedElements(
						processPage.getSelectedURIsWithDependencies().toArray());
			} else if (previousPage instanceof ServiceDeploymentChoicePage) {
				serviceTableViewer.setCheckedElements(new Object[0]);
			}
			dialogChanged();
		}
		super.setVisible(visible);
	}

	private void dialogChanged() {
		if (serviceTableViewer.getCheckedElements().length == 0) {
			updateStatus("Select services to deploy from the list below");
			return;
		}
		updateStatus(null);
	}

	private void updateStatus(String message) {
		setMessage(message);
		setPageComplete(message == null);
	}

	public void setProject(IProject project) {
		this.project = project;
	}

	public IProject getProject() {
		return project;
	}
	
	public List<URI> getSelectedURIs() {
		List<URI> toBeReturnedURIs = new ArrayList<URI>();
		for (Object checked : serviceTableViewer.getCheckedElements()) {
			toBeReturnedURIs.add((URI) checked);
		}
		return toBeReturnedURIs;
	}

	public List<URI> getSelectedDependencyURIs() {
		final List<String> filenames = new ArrayList<String>();
		for (URI uri : getSelectedURIs()) {
			final ServiceInfo info
					= infoRepository.findServiceInfoByURI(uri);
			if (info != null) {
				for (String filename : info.getResourceFilenames()) {
					if(!filenames.contains(filename)) {
						filenames.add(filename);
					}
				}
			}
		}
		return Activator.getDefault().getResourceURIs(filenames, getProject());
	}
	
	public List<URI> getSelectedResourcesURIs() {
		final List<URI> toReturnUris = new ArrayList<URI>();
		final List<String> filenames = new ArrayList<String>();
		for (URI uri : getSelectedURIs()) {
			toReturnUris.add(uri);
			final ServiceInfo info = infoRepository.findServiceInfoByURI(uri);
			if (info != null) {
				for (String filename : info.getResourceFilenames()) {
					if(!filenames.contains(filename)) {
						filenames.add(filename);
					}
				}
				toReturnUris.addAll(Activator.getDefault().getResourceURIs(filenames, getProject()));
				filenames.clear();
			}
		}
		return toReturnUris;
	}
	

}