/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
 **/
package org.spagic3.ui.deploymentservice.wizards;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipOutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ebpm.service.model.ServiceModelHelper;
import org.eclipse.ebpm.service.model.ServiceModelHelper.DeploySchemaHelper;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IWorkbench;
import org.spagic3.ui.deploymentservice.Activator;
import org.spagic3.ui.deploymentservice.actions.PreparePackagingArtifacts;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class ServiceDeploymentWizard extends Wizard implements IWizard {

	private ServiceDeploymentChoicePage choicePage;
	private ServiceDeploymentPage servicePage;
	private ProcessDeploymentPage processPage;
	private ISelection selection;
	private boolean showProjectList;
	private Document doc;
	private List<File> files = new ArrayList<File>();
	private List<String> fileNames = new ArrayList<String>();
	private List<String> processFile = new ArrayList<String>();
	private List<String> processElements = new ArrayList<String>();

	public ServiceDeploymentWizard(boolean showProjectList) {
		super();
		setNeedsProgressMonitor(true);
		setHelpAvailable(false);
		this.showProjectList = showProjectList;
	}

	public void addPages() {
		choicePage = new ServiceDeploymentChoicePage(selection, showProjectList);
		addPage(choicePage);
		servicePage = new ServiceDeploymentPage(selection);
		addPage(servicePage);
		processPage = new ProcessDeploymentPage(selection);
		addPage(processPage);
	}

	public IWizardPage getNextPage(IWizardPage page) {
		if (page == choicePage) {
			return choicePage.isProcessDeploy() ? processPage : servicePage;
		} else if (page == servicePage) {
			return null;
		} else if (page == processPage) {
			servicePage.setCreateZipArchive(processPage.isCreateZipArchive());
			servicePage.setArchiveName(processPage.getArchiveName()!=null?processPage.getArchiveName():"ArchiveName.zip");
			servicePage.updateZipFields();
			return servicePage;
		}
		return null;
	}

	public IWizardPage getPreviousPage(IWizardPage page) {
		if (page == choicePage) {
			return null;
		} else if (page == servicePage) {
			return choicePage.isProcessDeploy() ? processPage : choicePage;
		} else if (page == processPage) {
			return choicePage;
		}
		return null;
	}

	public boolean isProcessDeploy() {
		return choicePage.isProcessDeploy();
	}
	
	public boolean isCopyProcess() {
		return !processPage.isCreateZipArchive();
	}
	
	public boolean isCopyServices() {
		return !servicePage.isCreateZipArchive();
	}

	public boolean canFinish() {
		IWizardPage page = getContainer().getCurrentPage();
		if (page == servicePage) {
			return servicePage.isPageComplete();
		}
		return false;
	}

	public boolean performFinish() {
		boolean processesOK = false;
		String nomeArchivio = null;
		if(!isCopyProcess() || !isCopyServices()){
			files = new ArrayList<File>();
			fileNames = new ArrayList<String>();
			try{	
		        DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
		        DocumentBuilder docBuilder = dbfac.newDocumentBuilder();
		        doc = docBuilder.newDocument();
		        Element root = doc.createElement("elementList");
	            doc.appendChild(root);
	    		Element config = doc.createElement("config");
	    		String componentName = null;
	    		if(isProcessDeploy()){
	    			if(!isCopyProcess()){
		    			List<String> nomi = getSelectedProcessesPoolNames();
		    			if(nomi.isEmpty() || nomi.size()>1)
		    				throw new Exception("Error in process selection, too many or too few. Select only ONE in order to create deploy zip archive");
		    			//Se ha superato il precedente controllo ci sar un solo nome
		    			for(String nome:nomi)
		    				componentName = nome;
	    			}
	    		}
	    		else{
	    			if(!isCopyServices()){
	    				List<URI> selectedURIs = servicePage.getSelectedURIs();
	    				if(selectedURIs.isEmpty() || selectedURIs.size()>1)
		    				throw new Exception("Error in service selection, too many or too few. Select only ONE in order to create deploy zip archive");
	    				
	    			}    			
	    		}		
	    		config.setAttribute("componentName", componentName);
	    		config.setAttribute("isProcess", new Boolean(isProcessDeploy()).toString());
	            root.appendChild(config);	            
			}catch (Exception e) {
				MessageDialog.openError(getShell(), "Error", e.getMessage());
				//System.out.println(e);
			}
		}
		if (isProcessDeploy()){
			if(isCopyProcess()){
				processesOK = deployProcesses();
				if (!processesOK){
					MessageDialog.openError(getShell(), "Error In Process Deployment", "The wizard will be stopped");
					return this.performCancel();
				}
			}
			else{
				nomeArchivio = processPage.getArchiveName();
				processesOK = copyProcessesForZip();
				if (!processesOK){
					MessageDialog.openError(getShell(), "Error In Process Deployment", "The wizard will be stopped");
					return this.performCancel();
				}
			}
		}else{
			processesOK = true;
		}
		
		boolean servicesOK;
		if (isCopyServices())
			servicesOK = deployServices();
		else{
			nomeArchivio = servicePage.getArchiveName();
			servicesOK = copyServicesForZip();
		}
		if (!servicesOK){
			MessageDialog.openError(getShell(), "Error In Services Deployment", "The wizard will be stopped");
			return this.performCancel();
		}
		if ((processesOK && !isCopyProcess()) || 
			(servicesOK && !isCopyServices())) {
			try{
				File lista = File.createTempFile("processElementsList", ".xml");
				lista.deleteOnExit();
				writeXmlFile(lista);
				files.add(lista);
				fileNames.add("processElementsList.xml");				
				createZip(files, fileNames, nomeArchivio, choicePage.getProject().getFolder("Deployables").getLocationURI().getPath().toString());
				lista.delete();
				//Refresh project Folder
				choicePage.getProject().getFolder("Deployables").refreshLocal(IProject.DEPTH_INFINITE, null);
			}catch (Exception e) {
				MessageDialog.openError(getShell(), "Error", e.getMessage());
				//System.out.println(e);
			}
		}
		return processesOK && servicesOK;
	}
	
	private void writeXmlFile(File file) {
	    try {
	        Source source = new DOMSource(doc);
	        Result result = new StreamResult(file);
	        Transformer xformer = TransformerFactory.newInstance().newTransformer();
	        xformer.transform(source, result);
	    } catch (Exception e) {
	    	MessageDialog.openError(getShell(), "Error", e.getMessage());
	    	//System.out.println(e);
	    }
	}

	protected boolean runOperation(IRunnableWithProgress op) {
		try {
			getContainer().run(true, false, op);
			return true;
		} catch (InterruptedException e) {
			return false;
		} catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException.getMessage());
			return false;
		}
	}
	
	protected List<String> getSelectedProcessesPoolNames(){
		final List<URI> selectedURIs = servicePage.getSelectedURIs();
		
		List<String> processNames = new ArrayList<String>();
		ServiceInfoRepository repo = Activator.getDefault().getServiceInfoRepository(servicePage.getProject());
		for (URI uri : selectedURIs){
			ServiceInfo info = repo.findServiceInfoByURI(uri);
			if (info.getProcess() != null && info.getProcess() != ""){
				processNames.add(info.getProcess());
			}
		}
		return processNames;
	
	}
	
	protected boolean copyProcessesForZip(){
		List<URI> processRelatedURIs = new ArrayList<URI>();
		
		List<String> selectedProcessNames = getSelectedProcessesPoolNames();
			
			for (URI bpmnURI : processPage.getSelectedBpmnURIs()) {
				PreparePackagingArtifacts packagingArtifacts = new PreparePackagingArtifacts();
				final IFile[] bpmnFiles = ResourcesPlugin.getWorkspace().getRoot().findFilesForLocationURI(bpmnURI);
				
				if (bpmnFiles != null && bpmnFiles[0] != null) {
					
					packagingArtifacts.setBpmnFile(bpmnFiles[0]);
					packagingArtifacts.selectedProcessNames(selectedProcessNames);
					packagingArtifacts.doPackaging(processRelatedURIs);
					
				}
				if (packagingArtifacts.isErrors())
					return false;
			}
		
		final List<URI> fprocessRelatedURIs = processRelatedURIs;
		
		String dstDirName = "processes";
		File dstDir = new File(dstDirName);
		
		for (URI uri : fprocessRelatedURIs) {
			final File srcFile = new File(uri);
			files.add(srcFile);
			final String filename = srcFile.getName();
			int dotIndex = filename.lastIndexOf(".");
			final String extension = dotIndex == -1 ? "" : filename.substring(dotIndex + 1);			
			final File dstSubDir = new File(dstDir,extension);
			final File dstFile = new File(dstSubDir, filename);
			fileNames.add(dstFile.getPath().replace("\\", "/"));
			try{
				processFile.add(dstFile.getPath());
				processElements.add(dstFile.getPath());
			}catch (Exception e) {
				System.out.println(e);
				return false;
			}finally{
				dstSubDir.delete();
				dstFile.delete();
			}
		}	
		dstDir.delete();

		return true;
	}
  
	protected boolean deployProcesses(){
		List<URI> processRelatedURIs = new ArrayList<URI>();
		
		List<String> selectedProcessNames = getSelectedProcessesPoolNames();		
			for (URI bpmnURI : processPage.getSelectedBpmnURIs()) {
				PreparePackagingArtifacts packagingArtifacts = new PreparePackagingArtifacts();
				final IFile[] bpmnFiles = ResourcesPlugin.getWorkspace().getRoot().findFilesForLocationURI(bpmnURI);
				
				if (bpmnFiles != null && bpmnFiles[0] != null) {
					
					packagingArtifacts.setBpmnFile(bpmnFiles[0]);
					packagingArtifacts.selectedProcessNames(selectedProcessNames);
					packagingArtifacts.doPackaging(processRelatedURIs);
					
				}
				if (packagingArtifacts.isErrors())
					return false;
			}
		
		final List<URI> fprocessRelatedURIs = processRelatedURIs;
		
		IRunnableWithProgress opDeployPackaging = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					doFinishPackageDeployment(fprocessRelatedURIs, monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
		
		return runOperation(opDeployPackaging);
	}
	
	protected boolean copyServicesForZip(){
		final List<URI> resourcesURIs = servicePage.getSelectedResourcesURIs();

		ServiceModelHelper serviceModelHelper = null;
		try {
			serviceModelHelper = new ServiceModelHelper();
		} catch (Exception e) {
			System.out.println(e);
			return false;
		}
		
		Element el = doc.createElement("connectors");
        doc.getElementsByTagName("elementList").item(0).appendChild(el);
		el = doc.createElement("services");
        doc.getElementsByTagName("elementList").item(0).appendChild(el);
        Element previousNode = null;
		
		for (URI uri : resourcesURIs) {
			final File srcFile = new File(uri);
			files.add(srcFile);
			final String filename = srcFile.getName();
			int dotIndex = filename.lastIndexOf(".");
			final String extension = dotIndex == -1 ? "" : filename.substring(dotIndex + 1);
			final DeploySchemaHelper schemaHelper = serviceModelHelper != null ? serviceModelHelper
					.getDeploySchemaByExtension(extension)
					: null;
			final File dstSubDir = schemaHelper != null ? new File(schemaHelper.getPath()) : null;
			final File dstFile = new File(dstSubDir, filename);
			fileNames.add(dstFile.getPath().replace("\\", "/"));
			try{
				if(extension.equals("connector") || extension.equals("service")){
					el = doc.createElement(extension);
					el.setAttribute("name", dstFile.getPath().replace("\\", "/"));
					doc.getElementsByTagName(extension+"s").item(0).appendChild(el);
					previousNode = el;
					if(filename.substring(0, dotIndex).contains("Orchestration")){
						String poolName = filename.substring(0, dotIndex).substring(0, filename.indexOf("Orchestration"));
						for(String path : processElements) {
							path = path.replace("\\", "/");
							String[] parts = path.split("/");
							String name = parts[2];
							String nameNoExtension = name.substring(0,name.lastIndexOf("."));
							if(nameNoExtension.equals(poolName) || nameNoExtension.equals(poolName+"_prox")){
								Element elem = doc.createElement("resource");
								elem.setAttribute("name", path);
								previousNode.appendChild(elem);
							}
						}
					}
				}else{
					el = doc.createElement("resource");
					el.setAttribute("name", dstFile.getPath().replace("\\", "/"));
					previousNode.appendChild(el);
				}
			}catch (Exception e) {
				System.out.println(e);
				return false;
			}finally{
				dstSubDir.delete();
				dstFile.delete();
			}
		}
		
		return true;
	}
	
	private void createZip(List<File> files, List<String> names, String archiveName, String projectPath) {
    	final int BUFFER = 2048;
    	try {
          BufferedInputStream origin = null;
          FileOutputStream dest = new FileOutputStream(projectPath+"/"+archiveName);
          //FileOutputStream dest = new FileOutputStream(projectPath+"/"+projectName+".zip");
          ZipOutputStream out = new ZipOutputStream(new BufferedOutputStream(dest));
          

          byte data[] = new byte[BUFFER];

          for (Iterator it1 = names.iterator(), it2 = files.iterator(); it1.hasNext();) {
        	  boolean flagOk = true;
        	  String name = (String) it1.next();
        	  File file = (File) it2.next();   
        	  FileInputStream fi = new FileInputStream(file);
        	  origin = new BufferedInputStream(fi, BUFFER);
              ZipEntry entry = new ZipEntry(name);
              try {
            	  out.putNextEntry(entry);
              }catch (ZipException e) {
				// Do nothing the resource is already in the zip
            	  flagOk = false;
              }
              if(flagOk){
	              int count;
	              while((count = origin.read(data, 0, BUFFER)) != -1) {
	            	  out.write(data, 0, count);
	              }
              }
              origin.close();
          }
          out.close();
    	} catch(Exception e) {
    		MessageDialog.openError(getShell(), "Error", e.getMessage());
    		//System.out.println(e);
        }
    }	
	
	protected boolean deployServices(){
		final List<URI> selectedURIs = servicePage.getSelectedURIs();
		final List<URI> resourceURIs = servicePage.getSelectedDependencyURIs();

		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					doFinish(resourceURIs, selectedURIs, monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
			
		return runOperation(op);
			
	}

	private void copyFile(File srcFile, File dstFile) {
		try {
			final FileChannel srcChannel = new FileInputStream(srcFile).getChannel();
			final FileChannel dstChannel = new FileOutputStream(dstFile).getChannel();
			dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
			srcChannel.close();
			dstChannel.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void doFinish(List<URI> resourceURIs, List<URI> selectedURIs,
			IProgressMonitor monitor) throws CoreException {
		monitor.beginTask("Deploying services... ", 1);
		IPreferenceStore preferences = SpagicUtilsActivator.getDefault().getPreferenceStore();

		String dstDirName = preferences.getString(SpagicPreferenceConstants.P_DEPLOY_SERVICE_TARGET_DIR);
		if (dstDirName == null || "".equals(dstDirName)) {
			getShell().getDisplay().asyncExec(new Runnable() {
				public void run() {
					MessageDialog.openError(getShell(), "Error",
							"Deployment Path is empty");
				}
			});
			monitor.worked(1);
			return;
		}

		File dstDir = new File(dstDirName);
		if (!dstDir.exists()) {

			getShell().getDisplay().asyncExec(new Runnable() {
				public void run() {
					MessageDialog.openError(getShell(), "Error", "file not found");
				}
			});
			monitor.worked(1);
			return;
		}

		List<URI> toCopyURIs = new ArrayList<URI>();
		toCopyURIs.addAll(resourceURIs);
		toCopyURIs.addAll(selectedURIs);
		ServiceModelHelper serviceModelHelper;
		try {
			serviceModelHelper = new ServiceModelHelper();
		} catch (Exception e) {
			throw new CoreException(new Status(IStatus.ERROR, Activator.PLUGIN_ID, IStatus.ERROR, e.getMessage(), null));
		}
		for (URI uri : toCopyURIs) {
			final File srcFile = new File(uri);
			final String filename = srcFile.getName();
			int dotIndex = filename.lastIndexOf(".");
			final String extension = dotIndex == -1 ? "" : filename.substring(dotIndex + 1);
			final DeploySchemaHelper schemaHelper = serviceModelHelper != null ? serviceModelHelper
					.getDeploySchemaByExtension(extension)
					: null;
			final File dstSubDir = schemaHelper != null ? new File(dstDir, schemaHelper.getPath()) : dstDir;
			if (!dstSubDir.exists()) {
				dstSubDir.mkdirs();
			}
			final File dstFile = new File(dstSubDir, filename);
			if (dstFile != null) {
				copyFile(srcFile, dstFile);
			}
		}
		monitor.worked(1);
	}
	
	
	private void doFinishPackageDeployment(List<URI> processRelatedURIs,
			IProgressMonitor monitor) throws CoreException {
		monitor.beginTask("Deploying Process Files and Related Resources... ", 1);
		IPreferenceStore preferences = SpagicUtilsActivator.getDefault().getPreferenceStore();

		String dstDirName = preferences.getString(SpagicPreferenceConstants.P_DEPLOY_SERVICE_TARGET_DIR);
		
		dstDirName += File.separator + "processes";
		
		if (dstDirName == null || "".equals(dstDirName)) {
			getShell().getDisplay().asyncExec(new Runnable() {
				public void run() {
					MessageDialog.openError(getShell(), "Error",
							"Path is empty");
				}
			});
			monitor.worked(1);
			return;
		}

		File dstDir = new File(dstDirName);
		if (!dstDir.exists()) {

			dstDir.mkdirs();
		}

		List<URI> toCopyURIs = new ArrayList<URI>();
		toCopyURIs.addAll(processRelatedURIs);
		
		
		for (URI uri : toCopyURIs) {
			final File srcFile = new File(uri);
			final String filename = srcFile.getName();
			int dotIndex = filename.lastIndexOf(".");
			final String extension = dotIndex == -1 ? "" : filename.substring(dotIndex + 1);
			
			
			final File dstSubDir = new File(dstDir,extension);
			if (!dstSubDir.exists()) {
				dstSubDir.mkdirs();
			}
			final File dstFile = new File(dstSubDir, filename);
			if (dstFile != null) {
				copyFile(srcFile, dstFile);
			}
		}
		monitor.worked(1);
	}

	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}
}