package org.spagic3.utils;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ImFactory;
import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.Service;
import org.eclipse.stp.im.ServiceCollection;
import org.eclipse.stp.im.Step;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.constants.ImConstants;

public class LoopContext {
	
	private Step originalLoopStep = null;	
	private Step loopSplitterStep = null;
	private Step loopCheckStep = null;
	private Step loopAggregatorStep = null;
	private Step loopIncrementIndexStep = null;
	
	private Step loopFirstStep = null;
	private List<Step> loopEndStep = null;
	private org.eclipse.stp.im.Process loopSubProcess = null;
	
	private String nameOfCurrentIndexVariable = null;
	private String nameOfLastIndexVariable = null;
	
	

	public LoopContext(ImFactory factory, 
					Step originalLoopStep, 
					Step loopSplitterStep,
					Step loopCheckStep, 
					Step loopAggregatorStep,
					Step loopIncrementIndexStep) {
		
		super();
		this.loopEndStep = new ArrayList<Step>();
		this.originalLoopStep = originalLoopStep;
		this.loopCheckStep = loopCheckStep;
		
		
		setPropertyOnStep(factory, this.loopCheckStep, ImConstants.IS_EXCLUSIVE_GATEWAY, "true");
		
		this.nameOfCurrentIndexVariable = originalLoopStep.getName() + "_idx_current";
		this.nameOfLastIndexVariable = originalLoopStep.getName() + "_idx_last";
		
		// Copy the properties of the OriginalStep to generated Steps 
		this.loopSplitterStep = loopSplitterStep;
		
		setPropertyOnStep(factory, this.loopSplitterStep, "CurrentIndexVariableName", nameOfCurrentIndexVariable);
		setPropertyOnStep(factory, this.loopSplitterStep, "LastIndexVariableName", nameOfLastIndexVariable);
		
		// Copy the properties of the OriginalStep to generated Steps 
		this.loopAggregatorStep = loopAggregatorStep;
		
		setPropertyOnStep(factory, this.loopAggregatorStep, "CurrentIndexVariableName", nameOfCurrentIndexVariable);
		setPropertyOnStep(factory, this.loopAggregatorStep, "LastIndexVariableName", nameOfLastIndexVariable);
		
		// Copy the properties of the OriginalStep to generated Steps 
		this.loopIncrementIndexStep = loopIncrementIndexStep;
	
		setPropertyOnStep(factory, this.loopIncrementIndexStep, "CurrentIndexVariableName", nameOfCurrentIndexVariable);
		setPropertyOnStep(factory, this.loopIncrementIndexStep, "LastIndexVariableName", nameOfLastIndexVariable);
		
		
		for (String key : originalLoopStep.getProperties().keySet()){
			Property p = originalLoopStep.getProperties().get(key);
			
			if (p instanceof BasicProperty){
				String value = ((BasicProperty)p).getValue();
				setPropertyOnStep(factory, this.loopSplitterStep, key, value);
				setPropertyOnStep(factory, this.loopAggregatorStep, key, value);
				setPropertyOnStep(factory, this.loopIncrementIndexStep, key, value);
				
			}
		}
		
		System.out.println("----- FINE -----");
	}

	public Step getOriginalLoopStep() {
		return originalLoopStep;
	}

	public void setOriginalLoopStep(Step originalLoopStep) {
		this.originalLoopStep = originalLoopStep;
	}

	public Step getLoopSplitterStep() {
		return loopSplitterStep;
	}

	public void setLoopSplitterStep(Step loopSplitterStep) {
		this.loopSplitterStep = loopSplitterStep;
	}

	public Step getLoopCheckStep() {
		return loopCheckStep;
	}

	public void setLoopCheckStep(Step loopCheckStep) {
		this.loopCheckStep = loopCheckStep;
	}

	public Step getLoopAggregatorStep() {
		return loopAggregatorStep;
	}

	public void setLoopAggregatorStep(Step loopAggregatorStep) {
		this.loopAggregatorStep = loopAggregatorStep;
	}

	public Step getLoopIncrementIndexStep() {
		return loopIncrementIndexStep;
	}

	public void setLoopIncrementIndexStep(Step loopIncrementIndexStep) {
		this.loopIncrementIndexStep = loopIncrementIndexStep;
	}

	public Step getLoopFirstStep() {
		return loopFirstStep;
	}

	public void setLoopFirstStep(Step loopFirstStep) {
		this.loopFirstStep = loopFirstStep;
	}

	public List<Step> getLoopEndStep() {
		return loopEndStep;
	}

	public void addLoopEndStep(Step loopEndStep) {
		this.loopEndStep.add(loopEndStep);
	}
	
	public String getNameOfCurrentIndexVariable() {
		return nameOfCurrentIndexVariable;
	}

	public void setNameOfCurrentIndexVariable(String nameOfCurrentIndexVariable) {
		this.nameOfCurrentIndexVariable = nameOfCurrentIndexVariable;
	}

	public String getNameOfLastIndexVariable() {
		return nameOfLastIndexVariable;
	}

	public void setNameOfLastIndexVariable(String nameOfLastIndexVariable) {
		this.nameOfLastIndexVariable = nameOfLastIndexVariable;
	}
	
	public org.eclipse.stp.im.Process  getLoopSubProcess() {
		return loopSubProcess;
	}

	public void setLoopSubProcess(org.eclipse.stp.im.Process loopSubProcess) {
		this.loopSubProcess = loopSubProcess;
	}
	
	public void setPropertyOnStep(ImFactory factory, Step step,
			String propertyName, String propertyValue) {
				BasicProperty newProperty = factory.createBasicProperty();
				newProperty.setKey(propertyName);
				newProperty.setValue(propertyValue);
				step.getProperties().put(propertyName, newProperty);
	}
	
	public Service findServiceName(StpIntermediateModel im, String name) {
		ServiceCollection sc = im.getServiceCollection();
		for (Service aService : sc.getServices() ) {
			if (aService.getServiceName().equalsIgnoreCase(name)){
				return aService;
			}
		}
		return null;
	}
	
}
