/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.utils.preference;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.spagic3.utils.SpagicUtilsActivator;

/**
 * This class represents a preference page that
 * is contributed to the Preferences dialog. By 
 * subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows
 * us to create a page that is small and knows how to 
 * save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They
 * are stored in the preference store that belongs to
 * the main plug-in class. That way, preferences can
 * be accessed directly via the preference store.
 */

public class SpagicBPELPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	private Map<String, String> dbUrls = new HashMap<String, String>();
	private Map<String, String> dbDrivers = new HashMap<String, String>();
	private Map<String, String> dbDialects = new HashMap<String, String>();
	private StringFieldEditor dbUrlStringFieldEditor = null;
	private StringFieldEditor dbDriverStringFieldEditor = null;
	private StringFieldEditor dbDialectStringFieldEditor  = null;
	
	public SpagicBPELPreferencePage() {
		
		super(GRID);
		setPreferenceStore(SpagicUtilsActivator.getDefault().getPreferenceStore());
		setDescription("");
		
		dbUrls = new HashMap<String, String>();
		dbUrls.put("H2", "jdbc:h2:tcp://localhost/~/spagic");
		dbUrls.put("MySql", "jdbc:mysql://localhost:3306/spagic");
		dbUrls.put("PostgreSQL","jdbc:postgresql://localhost/spagic");
		dbUrls.put("Oracle", "jdbc:oracle:thin:@localhost:1521:spagic");
		
		dbDrivers = new HashMap<String, String>();
		dbDrivers.put("H2", "org.h2.Driver");
		dbDrivers.put("MySql", "com.mysql.jdbc.Driver");
		dbDrivers.put("PostgreSQL", "org.postgresql.Driver");
		dbDrivers.put("Oracle", "oracle.jdbc.driver.OracleDriver");
		
		dbDialects = new HashMap<String, String>();
		dbDialects.put("H2", "org.hibernate.dialect.H2Dialect");
		dbDialects.put("MySql", "org.hibernate.dialect.MySQLInnoDBDialect" );
		dbDialects.put("PostgreSQL", "org.hibernate.dialect.PostgreSQLDialect");
		dbDialects.put("Oracle",  "org.hibernate.dialect.OracleDialect");
		
		
	
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {
		
		GridLayout layoutPreference = new GridLayout();
		layoutPreference.numColumns = 1;
		
		Composite aComposite = new Composite(getFieldEditorParent(), SWT.NONE);
		aComposite.setLayout(layoutPreference);
		
		
		
		
		Group bpelGroup = new Group(
				aComposite, 
				SWT.SHADOW_ETCHED_IN);
		GridData gd1 = new GridData(GridData.FILL_HORIZONTAL);
		bpelGroup.setLayoutData(gd1);
		bpelGroup.setText("ODE Engine");
		
		
		Group dbGroup = new Group(
				aComposite, 
				SWT.SHADOW_ETCHED_IN);
		GridData gd2 = new GridData(GridData.FILL_HORIZONTAL);
		dbGroup.setLayoutData(gd2);
		dbGroup.setText("Metadatabase");
		

	
		addField(new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_ODE_URL_DEPLOY, "ODE deployment service url", bpelGroup));
		addField(new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_ODE_URL_PROCESS_MANAGEMENT, "ODE process management url", bpelGroup));
		
		
		
//		
		
		dbUrlStringFieldEditor = new StringFieldEditor(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_URL, "Spagic DB URL", dbGroup);
		dbDriverStringFieldEditor = new StringFieldEditor(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_DRIVER, "Spagic DB DRIVER", dbGroup);
		dbDialectStringFieldEditor = new StringFieldEditor(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_HIB_DIALECT, "Spagic DB DIALECT", dbGroup);
		addField(dbUrlStringFieldEditor);
		addField(dbDriverStringFieldEditor);
		
		addField(new StringFieldEditor(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_USER, "Spagic DB USER", dbGroup));
		addField(new StringFieldEditor(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_PASSWORD, "Spagic DB PASSWORD", dbGroup));
		addField(dbDialectStringFieldEditor);
		addField(new BooleanFieldEditor(SpagicPreferenceConstants.P_BPEL_SPAGIC_DB_SHOW_SQL,"Show sql",dbGroup));
	
		
	}

	/** (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		
	}

	@Override
	protected void contributeButtons(Composite parent) {
		super.contributeButtons(parent);
		createButtonForDatabase("H2", parent);
		createButtonForDatabase("MySql", parent);
		createButtonForDatabase("PostgreSQL", parent);
		createButtonForDatabase("Oracle", parent);
		
	}
	
	protected void createButtonForDatabase(String db, Composite parent){
		int widthHint = convertHorizontalDLUsToPixels(IDialogConstants.BUTTON_WIDTH);
		final Button dbButton = new Button(parent, SWT.PUSH);
		dbButton.setText(db);
		Dialog.applyDialogFont(dbButton);
		GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		Point mySqlButtonSize = dbButton.computeSize(SWT.DEFAULT, SWT.DEFAULT,
				true);
		data.widthHint = Math.max(widthHint, mySqlButtonSize.x);
		dbButton.setLayoutData(data);
		final String databaseText = db;
		dbButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				String dbText = dbButton.getText();
				dbUrlStringFieldEditor.setStringValue(dbUrls.get(dbText));
				dbDriverStringFieldEditor.setStringValue(dbDrivers.get(dbText));
				dbDialectStringFieldEditor.setStringValue(dbDialects.get(dbText));
			}
		});
	}
}